/*---------------------------------------------------------------------------
 *  Copyright (C) 2003-2006 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 *
 * This file contains function definitions for the built-in ROM functions 
 * of the Dallas Semiconductor 400 processor.  This file is intended for use 
 * with the Keil MicroVision (uVision) C compiler.
 *
 * ---------------------------------------------------------------------------
 */
#ifndef __rom400_init_
#define __rom400_init_

#pragma SAVE
#pragma REGPARMS

/** \file rom400_init.h
 *  \brief ROM Initialization functions in the DS80C400 ROM
 *
 *  This library contains functions for initializing the functionality
 *  in the ROM.  Note that the preferred way of initializing the ROM is
 *  to simply call the <i>#init_rom</i> function.  However, you can also
 *  initialize the various modules individually.  To do this, call 
 *  these functions in this order:
 *     -# <i> #init_clearXSEG </i>
 *     -# <i> #init_copyivt </i>
 *     -# <i> #init_redirect </i>
 *     -# <i> #init_sched </i>
 *     -# <i> #init_mm </i>
 *     -# <i> #init_km </i>
 *     -# <i> #init_ow </i>
 *     -# <i> #init_network </i>
 *     -# <i> #init_eth </i>
 *     -# <i> #init_sockets </i>
 *     -# <i> #init_tick </i>
 *     -# <i> #task_genesis </i> [in the process scheduler library]
 *     -# <i> #init_enableinterrupts </i>
 *
 *  For detailed information on the DS80C400 please see the
 *  <a href="http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf">
 *  High-Speed Microcontroller User's Guide: DS80C400 Supplement</a>.
 *
 *  Functions in this library should only be called once on startup.
 *  The safety of calling these functions from multiple processes at
 *  the same time is irrelevant.
 *  
 */

/** Version number associated with this header file.  Should be the same as
 * the version number returned by the <i>#init_version</i> function.
 * \sa #init_version */
#define ROM400_INIT_VERSION         19


/**
 *  Macro that allows the use of a define to determine whether or not
 * to call the function <i>#init_usekeilmonitor</i>.  This macro can
 * be called after calling <i>#init_rom</i>, and will correct some
 * monitor configuration details that are destroyed when <i>#init_rom</i>
 * is called.
 * 
 * \sa       #init_rom
 * \sa       #init_usekeilmonitor
 * 
 */
#ifdef MONITOR
  #define USE_KEIL_MONITOR init_usekeilmonitor();
#else
  #define USE_KEIL_MONITOR
#endif


/**
 * 1-Wire divisor value for operating frequencies greater than 3 MHz but less than 4 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_3MHZ    0x01      // 00000001B

/**
 * 1-Wire divisor value for operating frequencies greater than 4 MHz but less than 5 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_4MHZ    0x08      // 00001000B

/**
 * 1-Wire divisor value for operating frequencies greater than 5 MHz but less than 6 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_5MHZ    0x02      // 00000010B

/**
 * 1-Wire divisor value for operating frequencies greater than 6 MHz but less than 7 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_6MHZ    0x05      // 00000101B

/**
 * 1-Wire divisor value for operating frequencies greater than 7 MHz but less than 8 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_7MHZ    0x03      // 00000011B

/**
 * 1-Wire divisor value for operating frequencies greater than 8 MHz but less than 10 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_8MHZ    0x0C      // 00001100B

/**
 * 1-Wire divisor value for operating frequencies greater than 10 MHz but less than 12 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_10MHZ   0x06      // 00000110B

/**
 * 1-Wire divisor value for operating frequencies greater than 12 MHz but less than 14 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_12MHZ   0x09      // 00001001B

/**
 * 1-Wire divisor value for operating frequencies greater than 14 MHz but less than 16 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_14MHZ   0x07      // 00000111B

/**
 * 1-Wire divisor value for operating frequencies greater than 16 MHz but less than 20 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_16MHZ   0x10      // 00010000B

/**
 * 1-Wire divisor value for operating frequencies greater than 20 MHz but less than 24 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_20MHZ   0x0A      // 00001010B

/**
 * 1-Wire divisor value for operating frequencies greater than 24 MHz but less than 28 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_24MHZ   0x0D      // 00001101B

/**
 * 1-Wire divisor value for operating frequencies greater than 28 MHz but less than 32 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_28MHZ   0x0B      // 00001011B

/**
 * 1-Wire divisor value for operating frequencies greater than 32 MHz but less than 40 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_32MHZ   0x14      // 00010100B

/**
 * 1-Wire divisor value for operating frequencies greater than 40 MHz but less than 48 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_40MHZ   0x0E      // 00001110B

/**
 * 1-Wire divisor value for operating frequencies greater than 48 MHz but less than 56 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_48MHZ   0x11      // 00010001B

/**
 * 1-Wire divisor value for operating frequencies greater than 56 MHz but less than 64 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_56MHZ   0x0F      // 00001111B

/**
 * 1-Wire divisor value for operating frequencies greater than 64 MHz but less than 80 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_64MHZ   0x18      // 00011000B

/**
 * 1-Wire divisor value for operating frequencies greater than 80 MHz but less than 96 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_80MHZ   0x12      // 00010010B

/**
 * 1-Wire divisor value for operating frequencies greater than 96 MHz but less than 112 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_96MHZ   0x15      // 00010101B

/**
 * 1-Wire divisor value for operating frequencies greater than 112 MHz but less than 128 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_112MHZ  0x13      // 00010011B

/**
 * 1-Wire divisor value for operating frequencies greater than 128 MHz.
 * \sa #init_ow
 */
#define INIT_DIVISOR_128MHZ  0x1C      // 00011100B

/*
OWM_DIVISOR_TABLE:
        ; Clock Rate (MHz)   Divisor to use
        db      3,           00001B
        db      4,           01000B
        db      5,           00010B
        db      6,           00101B
        db      7,           00011B
        db      8,           01100B
        db      10,          00110B
        db      12,          01001B
        db      14,          00111B
        db      16,          10000B
        db      20,          01010B
        db      24,          01101B
        db      28,          01011B
        db      32,          10100B
        db      40,          01110B
        db      48,          10001B
        db      56,          01111B
        db      64,          11000B
        db      80,          10010B
        db      96,          10101B
        db      112,         10011B
OWM_DIVISOR_TABLE_MAX:
        db      128,         11100B
*/


/**
 * Power fail reset status.
 * \sa #init_getbootstate
 */
#define INIT_POWERFAIL_RESET	0x08

/**
 * Watchdog reset status.
 * \sa #init_getbootstate
 */
#define INIT_WATCHDOG_RESET	0x10

/**
 * Crystal failure reset status.
 * \sa #init_getbootstate
 */
#define INIT_CRYSTALFAIL_RESET	0x20

/**
 * Used to calculate the default start address for the heap.
 * \sa #init_rom
 */
extern void HEAP_START(void);

/**
 * Defines the default start address for the heap.
 * \sa #init_rom
 */
#define DEFAULT_HEAP_START ((((long)&HEAP_START)&0x7fffffL)-0x10000L)


/**
 * \brief     Initializes the modules in the ROM.
 *
 * Initializes the network stack, memory manager, process scheduler, and 
 * other modules in the DS80C400 Silicon Software.  Calling this method is
 * the preferred way of initializing the ROM.
 *
 * Note that calling this function will cause the ROM to copy its own
 * interrupt table into memory.  If you have any interrupts installed
 * before calling this function (for instance, you use the Keil compilers
 * <i>interrupt</i> keyword to declare your function an interrupt handler), 
 * the entry in the interrupt table will be erased.
 *
 * <i>init_rom</i> prints status information to the serial port if serial 0
 * is set to use timer 2. If that is not desired, clear <i>tr2</i>.
 * <code>
 *   ...
 *   TR2 = 0;
 *   init_rom(...);
 *   TR2 = 1;
 *   ...
 * </code>
 *
 * <i>init_rom</i> will probe all available 1-Wire devices for an approximate
 * clock frequency and it will try to find a DS2502-E48 for an Ethernet MAC 
 * address. If no DS2502-E48 is present, you must use <i>init_setfrequency</i>
 * to specify a clock frequency, and you must modify startup.a51 to manually set
 * a MAC address.
 *
 * \param     mem_start_address The absolute beginning address for the heap.
 * \param     mem_end_address   The absolute ending address for the heap.
 * 
 * Use mem_start_address==0 to use the default settings for both start and end,
 * or pass a value to mem_start_address and use mem_end_address==0 to use 
 * the remaining memory in the same bank, or use valid values for
 * both addresses.
 * Make sure the heap does not conflict with the XDATA segment (adjustable in
 * project settings); you can examine the MAP file to determine the size of XDATA.
 * Also note that the reentrant stack starts at the top of the XDATA segment
 * (start address adjustable in startup400.a51).
 *
 * Start address examples...
 * <center><table>
 *   <tr>
 *     <td>mem_start_address</td>  <td>mem_end_address</td>     <td>actual start</td>     <td>actual end</td>      <td>size of heap</td>
 *   </tr>
 *   <tr>
 *     <td>0x000000</td>           <td>0x000000</td>           <td>0x002900</td>        <td>0x00FFFF</td>            <td>55040</td>
 *   </tr>
 *   <tr>
 *     <td><i>#DEFAULT_HEAP_START</i></td> <td>0x07FFFF</td>           <td>0x002900</td>        <td>0x07FFFF</td>           <td>513792</td>
 *   </tr>
 *   <tr>
 *     <td>0x010440</td>           <td>0x000000</td>           <td>0x010440</td>        <td>0x01FFFF</td>            <td>64448</td>
 *   </tr>
 *   <tr>
 *     <td>0x010440</td>           <td>0x07FFFF</td>           <td>0x010440</td>        <td>0x07FFFF</td>           <td>457663</td>
 *   </tr>
 * </table></center>
 *
 * \sa #DEFAULT_HEAP_START
 */
//---------------------------------------------------------------------------
void  init_rom(unsigned long mem_start_address, unsigned long mem_end_address);

/**
 * \brief     Starts the netboot functionality.  Note that this will negate
 *            any initialization that has already been performed.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_netboot(void);

/**
 * \brief     Copies the interrupt vector table into memory.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_copyivt(void);

/**
 * \brief     Sets up the redirect table for ROM redirected calls.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_redirect(void);

/**
 * \brief     Sets up the default task scheduler.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_sched(void);

/**
 * \brief     Clears system variables in external RAM.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * This function also sets the <b>EPFI</b> bit.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_clearXSEG(void);

/**
 * \brief    Initializes the heap.    
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \param     mem_start_address The absolute beginning address for the heap
 *                              (see <i>#init_rom</i> for a detailed discussion of
 *                              the input parameters).  
 *                              Unlike the <i>#init_rom</i> function, this function 
 *                              cannot accept <b>0</b> for default parameters.  The
 *                              start address must be specified. Use <i>#DEFAULT_HEAP_START</i>
 *                              to specify the default start address.
 * \param     mem_end_address   The absolute ending address for the heap
 *                              (see <i>#init_rom</i> for a detailed discussion of
 *                              the input parameters).
 *                              Unlike the <i>#init_rom</i> function, this function 
 *                              cannot accept <b>0</b> for default parameters.  The
 *                              end address must be specified.
 *
 * \sa        #init_rom
 * \sa        #DEFAULT_HEAP_START
 */
//---------------------------------------------------------------------------
void  init_mm(unsigned long mem_start_address, unsigned long mem_end_address);

/**
 * \brief     Initializes fast kernel memory.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_km(void);

/**
 * \brief     Initializes the internal 1-Wire.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \param DIVISOR Divisor value for given the DS80C400's operating frequency.
 *                The operating frequency is the oscillator adjusted by any 
 *                setting of the frequency multiplier (i.e. a 14 MHz oscillator with
 *                the clock doubler enabled should look for a divisor for 28 MHz)
 *                
 * \sa        #init_rom
 * \sa        #INIT_DIVISOR_3MHZ
 * \sa        #INIT_DIVISOR_4MHZ
 * \sa        #INIT_DIVISOR_5MHZ
 * \sa        #INIT_DIVISOR_6MHZ
 * \sa        #INIT_DIVISOR_7MHZ
 * \sa        #INIT_DIVISOR_8MHZ
 * \sa        #INIT_DIVISOR_10MHZ
 * \sa        #INIT_DIVISOR_12MHZ
 * \sa        #INIT_DIVISOR_14MHZ
 * \sa        #INIT_DIVISOR_16MHZ
 * \sa        #INIT_DIVISOR_20MHZ
 * \sa        #INIT_DIVISOR_24MHZ
 * \sa        #INIT_DIVISOR_28MHZ
 * \sa        #INIT_DIVISOR_32MHZ
 * \sa        #INIT_DIVISOR_40MHZ
 * \sa        #INIT_DIVISOR_48MHZ
 * \sa        #INIT_DIVISOR_56MHZ
 * \sa        #INIT_DIVISOR_64MHZ
 * \sa        #INIT_DIVISOR_80MHZ
 * \sa        #INIT_DIVISOR_96MHZ
 * \sa        #INIT_DIVISOR_112MHZ
 * \sa        #INIT_DIVISOR_128MHZ
 */
//---------------------------------------------------------------------------
void  init_ow(unsigned char DIVISOR);

/**
 * \brief     Initializes the network.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_network(void);

/**
 * \brief     Initializes the ethernet support.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_eth(void);

/**
 * \brief     Initializest the socket layer.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_sockets(void);

/**
 * \brief     Initializes the system timer.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_tick(void);

/**
 * \brief     Enables system interrupts.
 *
 * Note that calling <i>#init_rom</i> is the preferred way of initializing the ROM.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void  init_enableinterrupts(void);

/**
 * \brief     Performs initialization necessary for using the Keil Monitor.
 *
 * Performs initialization needed when using the Keil MON390 Monitor to debug
 * programs that access the DS80C400's ROM.  This function should be called
 * after calling <i>#init_rom</i>, and only if the monitor will be used.
 *
 * This file includes a macro <i>#USE_KEIL_MONITOR</i> which is defined to 
 * call this function if <i>MONITOR</i> is defined.  Use the following
 * code to make use of this macro:
 *
 * <code><pre>
 *        init_rom();
 *         USE_KEIL_MONITOR
 * </pre></code>
 * 
 * \sa        #init_rom
 * \sa        #USE_KEIL_MONITOR
 */
//---------------------------------------------------------------------------
void  init_usekeilmonitor(void);

//void  init_resetMII(void);

/**
 * \brief     Returns the version number of this initialization library.
 *
 * \return    Version number of this INIT library.
 */
//---------------------------------------------------------------------------
unsigned int init_version(void);


/**
 * \brief     Returns the boot status flags.
 *
 * The status flags are defined as follows:
 *    Status.3 (0x08) - Power Fail Reset                 INIT_POWERFAIL_RESET
 *    Status.4 (0x10) - Watchdog Reset                   INIT_WATCHDOG_RESET
 *    Status.5 (0x20) - Crystal Oscillator Failure Reset INIT_CRYSTALFAIL_RESET
 * All other bits are reserved, but not necessarily 0.
 *
 * \return    Status flags
 *
 * \sa #INIT_POWERFAIL_RESET
 * \sa #INIT_WATCHDOG_RESET
 * \sa #INIT_CRYSTALFAIL_RESET
 */
//---------------------------------------------------------------------------
unsigned char init_getbootstate(void);


/**
 * \brief     Sets the crystal frequency.
 * 
 * \param value Clock frequency in kHz * 5/60 (e.g. 1229 for a 14.7456 MHz crystal).
 *              The operating frequency is the oscillator adjusted by any 
 *              setting of the frequency multiplier (i.e. a 14 MHz oscillator with
 *              the clock doubler enabled should set 28 MHz)
 *
 * Note that this function has to be called before <i>#init_rom</i>.
 * Users should call the more friendly macro <i>#init_setfrequency</i>.
 *
 * \sa	      #task_settickreload
 * \sa        #init_rom
 * \sa	      #init_setfrequency
 */
//---------------------------------------------------------------------------
void init_setclock(unsigned int value);

/**
 * \brief Sets the crystal frequency.
 * 
 * \param clock Clock frequency in kHz (e.g. 14746 for a 14.7456 MHz crystal).
 *              The operating frequency is the oscillator adjusted by any 
 *              setting of the frequency multiplier (i.e. a 14 MHz oscillator with
 *              the clock doubler enabled should set 28 MHz)
 *
 * Note that this macro has to be called before <i>#init_rom</i>.
 *
 * \sa	      #task_settickreload
 * \sa        #init_rom
 * \sa        #init_setclock
 */
//---------------------------------------------------------------------------
#define init_setfrequency(clock) init_setclock(((clock)*5L)/60)

#pragma RESTORE
#endif
