/*---------------------------------------------------------------------------
 *  Copyright (C) 2003-2006 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 *
 * This file contains function definitions for the built-in ROM functions 
 * of the Dallas Semiconductor 400 processor.  This file is intended for use 
 * with the Keil MicroVision (uVision) C compiler.
 *
 * ---------------------------------------------------------------------------
 */
#ifndef __rom400_sock_
#define __rom400_sock_

#include <stdlib.h>


/** \file rom400_sock.h
 *  \brief Socket functions in the DS80C400 ROM
 *
 *  This library contains functions for TCP, UDP and Multicast sockets,
 *  as well as network configuration.  The functions in
 *  this library <b>are</b> safe to be called from multiple processes at
 *  the same time, with the exception of the function <i>#ping</i>.
 *  Both the traditional Berkeley style socket API and the 
 *  <i>synchronized</i> socket functions are supported (the Berkeley style
 *  API is supported through macros implemented by the synchronized functions).  
 *
 *  It is recommended that new applications use the Berkeley style API
 *  for portability.
 * 
 *  Note that in order to run at 100 Mbs, the DS80C400 must be running at
 *  least 25MHz.  This can be accomplished on the TINIm400 module by enabling
 *  the clock doubler.
 *  
 *  For detailed information on the DS80C400 please see the
 *  <a href="http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf">
 *  High-Speed Microcontroller User's Guide: DS80C400 Supplement</a>.
 */

/** Version number associated with this header file.  Should be the same as
 * the version number returned by the <i>#sock_version</i> function.
 * \sa #sock_version */
#define ROM400_SOCK_VERSION               12

/** Version number associated with this header file.  Should be the same as
 * the version number returned by the <i>#sock_version</i> function.
 * \sa #syn_version */
#define ROM400_SOCK_SYNCH_VERSION         ROM400_SOCK_VERSION


/** Argument to function <i>#socket</i> to create a UDP socket (same as <i>#SOCK_DGRAM</i>)
 * \sa #socket  */
#define SOCKET_TYPE_DATAGRAM     0
/** Argument to function <i>#socket</i> to create a TCP socket (same as <i>#SOCK_STREAM</i>)
 * \sa #socket  */
#define SOCKET_TYPE_STREAM       1
/** Argument to function <i>#socket</i> to create a UDP socket (same as <i>#SOCKET_TYPE_DATAGRAM</i>)
 * \sa #socket  */
#define SOCK_DGRAM               SOCKET_TYPE_DATAGRAM
/** Argument to function <i>#socket</i> to create a TCP socket (same as <i>#SOCKET_TYPE_STREAM</i>)
 * \sa #socket  */
#define SOCK_STREAM              SOCKET_TYPE_STREAM

/** IPv4 protocol family define */
#define PF_INET                  4

/** IPv4 family define, ignored by DS80C400 Silicon Software, but included for compatibility */
#define AF_INET      4

/** IPv6 family define, ignored by DS80C400 Silicon Software, but included for compatibility */
#define AF_INET6     6

/** Protocol ID define, ignored by DS80C400 Silicon Software, but included for compatibility */
#define IPPROTO_UDP  0


/** Argument for socket option.  Enables/disables Nagle algorithm.
 * \sa #getsockopt 
 * \sa #setsockopt */
#define TCP_NODELAY              0
/** Argument for socket option.  Ignored by DS80C400 ROM.
 * \sa #getsockopt 
 * \sa #setsockopt */
#define SO_LINGER                1
/** Argument for socket option.  Socket inactivity timeout.
 * \sa #getsockopt 
 * \sa #setsockopt */
#define SO_TIMEOUT               2
/** Argument for socket option.  Local binding address.
 * \sa #getsockopt 
 * \sa #setsockopt */
#define SO_BINDADDR              3

/** Flag for analyzing ethernet status.
 * \sa #getethernetstatus */
#define ETH_STATUS_LINK          1


/**
 * Structure for an IP address.  For a normal, IPv4 (4 byte) address, 
 * set the address in sin_addr[12,13,14,15], with the most significant
 * byte at sin_addr[12].  Notice the 3 byte <i>reserved</i> to deal with
 * the TNI native interface overhead.
 *
 */
struct sockaddr
{
    unsigned char  reserved[3];     ///< Overhead for TNI native interface.
    unsigned char  sin_addr[16];    ///< IP address. IPv4 address is in sin_addr[12-15] with MSB at sin_addr[12].
    unsigned short sin_port;        ///< 16 bit port number for the socket.
    unsigned char  sin_family;      ///< Ignored by DS80C400 implementation.
};

/**
 * Structure representing a 4 byte IPv4 address, for use with the
 * #sockaddr_in structure.
 *
 */
struct in_addr
{
    unsigned long s_addr;      ///< Address as an unsigned long (32 bits). 
};

/**
 * Structure representing a 16 byte IPv6 address.
 *
 */
struct in6_addr
{
    unsigned char s6_addr[16];    ///< IPv6 compatible address
};

/**
 * Alternate structure for an IP address.  For a normal, IPv4 (4 byte) address, 
 * set the address in sin_addr.s_addr, and set sin_zero to all 0's.  Notice the 
 * 3 byte <i>reserved</i> to deal with the TNI native interface overhead.
 *
 */
struct sockaddr_in
{
    unsigned char  reserved[3];     ///< Overhead for TNI native interface.
    unsigned char  sin_zero[12];    ///< Zeroes in IP address due to IPv6 support.
    struct in_addr sin_addr;        ///< IPv4 address structure.
    unsigned short sin_port;        ///< 16 bit port number for the socket.
    unsigned char  sin_family;      ///< Ignored by DS80C400 implementation.
};

/**
 * The ping return data structure
 */
struct pingdata {
  unsigned char reserved[3];              ///< Reserved field
  unsigned char ip_header[20];            ///< The IP header of the return packet
  unsigned char icmp_header[8];           ///< icmp_header - The ICMP header of the return packet
  unsigned char icmp_data[32];            ///< icmp_data - The ICMP data portion of the return packet (should be 0x20,0x21,0x22,...,0x3f)
};


/**
 * \brief    Converts a numeric address to a string
 *
 * Converts a numeric IP address to a presentable format as a 
 * null terminated string.  IPv4 addresses are formatted such as
 * in "192.0.1.1".  IPv6 addresses are formatted such as in 
 * "b803:8a11:0000:2121:fec5:0601:aa01:0102".  Note that the '::'
 * shortcut is <b>not</b> supported--a '0000' must be fully 
 * specified.
 *
 * \param    family    AF_INET or AF_INET6
 * \param    addr      pointer to numeric representation of IP address
 * \param    strptr    storage location for presentation string
 * \param    len       size of storage area for strptr
 *
 * \return   Reference to strptr, or NULL if the <i>family</i> is not recognized
 *           or if there is not enough space as declared by <i>len</i>
 *
 * \sa       #inet_pton
 */
//---------------------------------------------------------------------------
char* inet_ntop(int family, void* addr, char* strptr, size_t len);

/**
 * \brief    Converts a string to a numeric IP address
 *
 * Converts a string represenation of an IP address into numeric format.
 * IPv4 addresses are expected to be input in a format such as
 * in "192.0.1.1".  IPv6 addresses are expected to be formatted such 
 * as in "b8:03:8a:11:00:00:21:21:fe:c5:06:01:aa:01:01:02".
 *
 * \param    family    AF_INET or AF_INET6
 * \param    str       address string to translate
 * \param    addr      pointer to storage for numeric representation of IP address
 *
 * \return   1 for successful translation.  0 if the format was invalid, or 
 *           the <i>family</i> was not recognized.
 *
 * \sa       #inet_ntop
 */
//---------------------------------------------------------------------------
unsigned int inet_pton(int family, char* str, void* addr);

/**
 * \brief    Convert a number to network byte order
 *
 * Converts a word from host byte order to network byte order.
 * On the DS80C400, the orders are the same, so this function
 * does not alter the input data.  This function is included
 * for compatibility.
 *
 * \param    x    Input data to convert to network byte order
 *
 * \return   Input data converted to network byte order
 */
//---------------------------------------------------------------------------
#define htons(x) (x)

/**
 * \brief    Convert a number to host byte order
 *
 * Converts a word from network byte order to host byte order.
 * On the DS80C400, the orders are the same, so this function
 * does not alter the input data.  This function is included
 * for compatibility.
 *
 * \param    x    Input data to convert to network byte order
 *
 * \return   Input data converted to network byte order
 */
//---------------------------------------------------------------------------
#define ntohs(x) (x)

/**
 * \brief    Converts a string representing an IPv4 address to numeric form.
 *
 * Converts the input string into an IPv4 address suitable for setting
 * in a <i>#sockaddr_in</i> structure.
 *
 * \param    inet_string    IPv4 address in string form
 *
 * \return   Numberic IPv4 address
 *
 * \sa       #sockaddr_in
 */
//---------------------------------------------------------------------------
unsigned long inet_addr(char* inet_string);


/**
 * \brief     Create a network socket for TCP or UDP communication.
 *
 * Creates a socket for network communication.  This function returns
 * a socket handle, but has not specific local address assigned to it.
 * Note that this function calls <i>#task_gettaskid</i> through the 
 * function redirect table.
 *
 * \param     type     #SOCKET_TYPE_DATAGRAM or #SOCK_DGRAM for UDP, 
 *                     #SOCKET_TYPE_STREAM or #SOCK_STREAM for TCP
 *
 * \return    -1 for failure, or the socket handle (socket number)
 *
 * \sa        #bind
 * \sa        #connect
 * \sa        #closesocket
 */
//---------------------------------------------------------------------------
int  syn_socket(unsigned int type);

/**
 * \brief     Create a network socket for TCP or UDP communication.
 *
 * Creates a socket for network communication.  This function returns
 * a socket handle, but has not specific local address assigned to it.
 * Note that this function calls <i>#task_gettaskid</i> through the 
 * function redirect table.
 *
 * \param     domain   ignored
 * \param     type     #SOCKET_TYPE_DATAGRAM or #SOCK_DGRAM for UDP, 
 *                     #SOCKET_TYPE_STREAM or #SOCK_STREAM for TCP
 * \param     protocol ignored
 *
 * \return    0x0FFFF for failure, or the socket handle (socket number)
 *
 * \sa        #bind
 * \sa        #connect
 * \sa        #closesocket
 */
//---------------------------------------------------------------------------
#define socket(domain,type,protocol)         syn_socket((type))


/**
 * \brief     Set the IP address parameter for future datagram calls.
 *
 * In order to keep the functions in this library multi-process-safe,
 * datagram functions <i>#syn_sendto</i> and <i>#syn_recvfrom</i> cannot have
 * as many parameters as their traditional counterparts.  This function 
 * sets the pointer to the address structure that will be used as the address
 * parameter for functions <i>#syn_sendto</i> and <i>#syn_recvfrom</i>.
 *
 * Note that the Berkeley style API is now supported and is multi-process
 * safe, so user software should never have to call this function.
 *
 * \param     socket_num Socket number to set address for
 * \param     sending    Set to 0 if this is an address for receiving,
 *                       Set to 1 if this is an address for sending
 * \param     addr       Address structure that will be used in future calls
 *                       to <i>#syn_sendto</i> or <i>#syn_recvfrom</i>.
 *
 * \return    socket_num (for Macro purposes)
 *
 * \sa        #syn_sendto
 * \sa        #syn_recvfrom
 *
 */
//---------------------------------------------------------------------------
int  syn_setDatagramAddress(int socket_num, unsigned char sending, struct sockaddr* addr);

/**
 * \brief     Sends a UDP datagram to an address earlier specified by 
 *            a call to <i>#syn_setDatagramAddress</i>.
 *
 * Sends a UDP datagram to an address earlier specified by a call to
 * <i>#syn_setDatagramAddress</i>.  The success/failure code 
 * this function returns says nothing of if the packet was recieved by the 
 * target, only that the socket layer was able to push the data out.  The 
 * socket <i>socket_num</i> must have been created with a type 
 * <i>#SOCKET_TYPE_DATAGRAM</i>.
 * 
 * \param     socket_num      the socket handle to use to send a UDP packet
 * \param     length          the number of bytes to send in the datagram packet
 * \param     buffer          the data to send in the datagram packet
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #recvfrom
 * \sa        #socket
 * \sa        #syn_setDatagramAddress
 */
//---------------------------------------------------------------------------
int  syn_sendto(int socket_num, unsigned int length, void* buffer);

/**
 * \brief     Sends a UDP datagram to a specified address.
 *
 * Sends a UDP datagram to a specified address.  The success/failure code 
 * this function returns says nothing of if the packet was recieved by the 
 * target, only that the socket layer was able to push the data out.  The 
 * socket <i>socket_num</i> must have been created with a type 
 * <i>#SOCKET_TYPE_DATAGRAM</i>.
 * 
 * \param     socket_num      the socket handle to use to send a UDP packet
 * \param     buffer          the data to send in the datagram packet
 * \param     length          the number of bytes to send in the datagram packet
 * \param     flags           ignored
 * \param     address         the destination address and port
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #recvfrom
 * \sa        #socket
 */
//---------------------------------------------------------------------------
#define sendto(socket_num,buffer,length,flags,address,address_length)   syn_sendto(syn_setDatagramAddress((socket_num),1,(address)),(length),(buffer))

/**
 * \brief     Receive a UDP datagram.
 *
 * Receives a message on the specified socket, and stores the address that 
 * sent it in the address structure set by an earlier call to 
 * <i>#syn_setDatagramAddress</i>.  If no data is available,
 * <i>#syn_recvfrom</i> blocks subject to the <i>#SO_TIMEOUT</i> value.  The socket
 * <i>socket_num</i> must have been created with a type
 * <i>#SOCKET_TYPE_DATAGRAM</i>.  It is required to use <i>#syn_bind</i> to assign a 
 * local port to the socket, before receiving data.
 * <b>NOTE:</b> This function reads <b>up to</b> <i>length</i> bytes of a 
 * datagram. Any data not read in the datagram will be discarded.
 * 
 * \param     socket_num      the socket handle to use to wait for and read a UDP packet
 * \param     length          the maximum number of bytes to read from a datagram socket
 * \param     buffer          the location to write any data read from the datagram socket
 *
 * \return    The number of bytes read.  If the operation times out according
 *            to the <i>SO_TIMEOUT</i>, a value of -2 is returned.  If another
 *            error occurs, -1 is returned.
 *
 * \sa        #sendto
 * \sa        #socket
 * \sa        #bind
 */
//---------------------------------------------------------------------------
int  syn_recvfrom(int socket_num, unsigned int length, void* buffer);

/**
 * \brief     Receive a UDP datagram.
 *
 * Receives a message on the specified socket, and stores the address that 
 * sent it.  If no data is available,
 * <i>#recvfrom</i> blocks subject to the <i>#SO_TIMEOUT</i> value.  The socket
 * <i>socket_num</i> must have been created with a type
 * <i>#SOCKET_TYPE_DATAGRAM</i>.  It is required to use <i>#bind</i> to assign a 
 * local port to the socket, before receiving data.
 * <b>NOTE:</b> This function reads <b>up to</b> <i>length</i> bytes of a 
 * datagram. Any data not read in the datagram will be discarded.
 * 
 * \param     socket_num      the socket handle to use to wait for and read a UDP packet
 * \param     buffer          the location to write any data read from the datagram socket
 * \param     length          the maximum number of bytes to read from a datagram socket
 * \param     flags           ignored
 * \param     address         location to fill in the address and port of the sender
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    The number of bytes read.  If the operation times out according
 *            to the <i>SO_TIMEOUT</i>, a value of -2 is returned.  If another
 *            error occurs, -1 is returned.
 *
 * \sa        #sendto
 * \sa        #socket
 * \sa        #bind
 */
//---------------------------------------------------------------------------
#define recvfrom(socket_num,buffer,length,flags,address,address_length) syn_recvfrom(syn_setDatagramAddress((socket_num),0,(address)),(length),(buffer))


/**
 * \brief     Connects a TCP socket to a specified address.
 *
 * Connects to a specified address with a streaming socket.  This function can 
 * only be used once with each socket.  The socket <i>socket_num</i> must have 
 * been created with type <i>#SOCKET_TYPE_STREAM</i>.
 *
 * \param     socket_num      the socket handle to use to wait for and read a UDP packet
 * \param     address         IP address and port number to create a streaming connection to
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #socket
 */
//---------------------------------------------------------------------------
int  syn_connect(int socket_num, struct sockaddr* address);

/**
 * \brief     Connects a TCP socket to a specified address.
 *
 * Connects to a specified address with a streaming socket.  This function can 
 * only be used once with each socket.  The socket <i>socket_num</i> must have 
 * been created with type <i>#SOCKET_TYPE_STREAM</i>.
 *
 * \param     socket_num      the socket handle to use to wait for and read a UDP packet
 * \param     address         IP address and port number to create a streaming connection to
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #socket
 */
//---------------------------------------------------------------------------
#define connect(socket_num,address,address_length)        syn_connect((socket_num),(address))

/**
 * \brief     Binds a socket to a specified address.
 *
 * Assigns a local address and port (stored in the <i>address</i> parameter)
 * to a socket.  Binding a socket is necessary for server sockets.  For
 * client sockets, use <i>#bind</i> if a specific source port is desirable.
 *
 * Fill <i>address</I> with 0's (for sin_addr and sin_port) to bind to
 * any available local port.  Use <i>#getsockname</i> to discover 
 * which port the socket was bound to.
 *
 * <b>NOTE:</b> When binding a UDP socket, matching inbound UDP packets will
 * be queued up for the socket. Call <i>#recvfrom</i> periodically to avoid
 * the risk of running out of kernel memory.
 *
 * \param     socket_num      socket handle to bind to a local port number
 * \param     address         contains the local address (including port number)
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #listen
 * \sa        #getsockname
 * \sa        #recvfrom
 * \sa        #unbind
 */
//---------------------------------------------------------------------------
int  syn_bind(int socket_num, struct sockaddr* address);

/**
 * \brief     Binds a socket to a specified address.
 *
 * Assigns a local address and port (stored in the <i>address</i> parameter)
 * to a socket.  Binding a socket is necessary for server sockets.  For
 * client sockets, use <i>#bind</i> if a specific source port is desirable.
 *
 * Fill <i>address</I> with 0's (for sin_addr and sin_port) to bind to
 * any available local port.  Use <i>#getsockname</i> to discover 
 * which port the socket was bound to.
 *
 * <b>NOTE:</b> When binding a UDP socket, matching inbound UDP packets will
 * be queued up for the socket. Call <i>#recvfrom</i> periodically to avoid
 * the risk of running out of kernel memory.
 *
 * \param     socket_num      socket handle to bind to a local port number
 * \param     address         contains the local address (including port number)
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #listen
 * \sa        #getsockname
 * \sa        #recvfrom
 */
//---------------------------------------------------------------------------
#define bind(socket_num,address,address_length)           syn_bind((socket_num),(address))

/**
 * \brief     Tells a socket to listen for incoming connections.
 *
 * Tells the socket to listen for connections.  A queue of length <i>backlog</i>
 * is created for pending (un-<i>#accept</i>ed connections).  It is required to 
 * use <i>#bind</i> to assign a local port before calling <i>#listen</i>.  Use
 * <i>#accept</i> to move an incoming request to an established state, or wait
 * for incoming connections.
 *
 * \param     socket_num  socket handle that will listen for connections
 * \param     backlog     the maximum number of pending connections (max 16
 *                        for the DS80C400)
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #bind
 * \sa        #accept
 */
//---------------------------------------------------------------------------
int  listen(int socket_num, unsigned int backlog);

/**
 * \brief     Tells a socket to listen for incoming connections.
 *
 * Tells the socket to listen for connections.  A queue of length <i>backlog</i>
 * is created for pending (un-<i>#accept</i>ed connections).  It is required to 
 * use <i>#bind</i> to assign a local port before calling <i>#listen</i>.  Use
 * <i>#accept</i> to move an incoming request to an established state, or wait
 * for incoming connections.
 *
 * \param     socket_num  socket handle that will listen for connections
 * \param     backlog     the maximum number of pending connections (max 16
 *                        for the DS80C400)
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #bind
 * \sa        #accept
 */
//---------------------------------------------------------------------------
#define syn_listen(socket_num,backlog)           listen((socket_num),(backlog))

/**
 * \brief     Accepts TCP connections on the specified socket.
 * 
 * Accepts a TCP conection on the specified socket.  This function moves
 * the first pending connection request from the listen queue into the
 * established state, assigning a new local socket to the connection
 * for communication.  <i>#accept</i> blocks if there are no pending
 * incoming requests.  The socket <i>socket_num</i> must have been created 
 * with type <i>#SOCKET_TYPE_STREAM</i>, bound to an address using <i>#bind</i>, 
 * and given a listen queue by calling <i>#listen</i>.
 *
 * \param     socket_num      the handle of the socket that will wait for connections
 * \param     address         location to write remote address
 *
 * \return    New socket handle for communicating with remote 
 *            socket, or -1 for failure
 *
 * \sa        #socket
 * \sa        #bind
 * \sa        #listen
 */
//---------------------------------------------------------------------------
int  syn_accept(int socket_num, struct sockaddr* address);

/**
 * \brief     Accepts TCP connections on the specified socket.
 * 
 * Accepts a TCP conection on the specified socket.  This function moves
 * the first pending connection request from the listen queue into the
 * established state, assigning a new local socket to the connection
 * for communication.  <i>#accept</i> blocks if there are no pending
 * incoming requests.  The socket <i>socket_num</i> must have been created 
 * with type <i>#SOCKET_TYPE_STREAM</i>, bound to an address using <i>#bind</i>, 
 * and given a listen queue by calling <i>#listen</i>.
 *
 * \param     socket_num      the handle of the socket that will wait for connections
 * \param     address         location to write remote address
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    New socket handle for communicating with remote 
 *            socket, or 0x0FFFF for failure
 *
 * \sa        #socket
 * \sa        #bind
 * \sa        #listen
 */
//---------------------------------------------------------------------------
#define accept(socket_num,address,address_length)         syn_accept((socket_num),(address))

/**
 * \brief     Reads data from a TCP socket.
 *
 * Reads data from a TCP socket.  If there is no data available, <i>#recv</i>
 * blocks until there is data, subject to the value of <i>#SO_TIMEOUT</i>.
 * <b>NOTE:</b> This function reads <b>up to</b> <i>length</i> bytes. Call this function
 * repeatedly if you need to read a minimum number of bytes.
 *
 * \param     socket_num  handle of the streaming socket that will read data
 * \param     length      maximum amount of data to read
 * \param     buffer      location to write any data read
 *
 * \return    The number of bytes read.  If the operation times out according
 *            to the <i>SO_TIMEOUT</i>, a value of -2 is returned.  If another
 *            error occurs, -1 is returned.  If the socket was closed by the 
 *            other side, 0 is returned.
 *
 * \sa        #connect
 * \sa        #send
 */
//---------------------------------------------------------------------------
int  syn_recv(int socket_num, unsigned int length, void* buffer);

/**
 * \brief     Reads data from a TCP socket.
 *
 * Reads data from a TCP socket.  If there is no data available, <i>#recv</i>
 * blocks until there is data, subject to the value of <i>#SO_TIMEOUT</i>.
 * <b>NOTE:</b> This function reads <b>up to</b> <i>length</i> bytes. Call this function
 * repeatedly if you need to read a minimum number of bytes.
 *
 * \param     socket_num  handle of the streaming socket that will read data
 * \param     buffer      location to write any data read
 * \param     length      maximum amount of data to read
 * \param     flags       ignored
 *
 * \return    The number of bytes read.  If the operation times out according
 *            to the <i>SO_TIMEOUT</i>, a value of -2 is returned.  If another
 *            error occurs, -1 is returned.  If the socket was closed by the 
 *            other side, 0 is returned.
 *
 * \sa        #connect
 * \sa        #send
 */
//---------------------------------------------------------------------------
#define recv(socket_num,buffer,length,flags)         syn_recv((socket_num),(length),(buffer))

/**
 * \brief     Sends data to a TCP socket.
 *
 * Writes data to a TCP socket.  The return value of this function is only a 
 * local success/failure code, and may not necessarily detect transmission 
 * errors.
 *
 * \param     socket_num  handle of the streaming socket that will write data
 * \param     length      number of bytes to write
 * \param     buffer      location of data to write
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #connect
 * \sa        #recv
 */
//---------------------------------------------------------------------------
int  syn_send(int socket_num, unsigned int length, void* buffer);

/**
 * \brief     Sends data to a TCP socket.
 *
 * Writes data to a TCP socket.  The return value of this function is only a 
 * local success/failure code, and may not necessarily detect transmission 
 * errors.
 *
 * \param     socket_num  handle of the streaming socket that will write data
 * \param     buffer      location of data to write
 * \param     length      number of bytes to write
 * \param     flags       ignored
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #connect
 * \sa        #recv
 */
//---------------------------------------------------------------------------
#define send(socket_num,buffer,length,flags)         syn_send((socket_num),(length),(buffer))


/**
 * \brief     Get various socket options.
 *
 * Reads a number of supported socket options.  Data written into the
 * buffer depends on the requested socket option.
 *
 * <table>
 *   <tr> <td> <b>Name</b> </td> <td> <b>Description</b> </td> <td> <b>Data in buffer</b> </td> </tr>
 *   <tr> <td> TCP_NODELAY </td> <td> TCP Nagle Enable   </td> <td> 1 byte                </td> </tr>
 *   <tr> <td> SO_LINGER   </td> <td> Ignored            </td> <td> N/A                   </td> </tr>
 *   <tr> <td> SO_TIMEOUT  </td> <td> Inactivity timeout </td> <td> 4 bytes (milliseconds, MSB first) </td> </tr>
 *   <tr> <td> SO_BINDADDR </td> <td> Local socket IP    </td> <td> 16 bytes              </td> </tr>
 * </table>
 *
 * This function assumes there is enough room in <i>buffer</i> to store
 * the requested data.
 *
 * \param     socket_num  socket to get option information for
 * \param     name        option to get
 * \param     buffer      location where option data will be written
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #setsockopt
 */
//---------------------------------------------------------------------------
int  syn_getsockopt(int socket_num, unsigned int name, void* buffer);

/**
 * \brief     Get various socket options.
 *
 * Reads a number of supported socket options.  Data written into the
 * buffer depends on the requested socket option.
 *
 * <table>
 *   <tr> <td> <b>Name</b> </td> <td> <b>Description</b> </td> <td> <b>Data in buffer</b> </td> </tr>
 *   <tr> <td> TCP_NODELAY </td> <td> TCP Nagle Enable   </td> <td> 1 byte                </td> </tr>
 *   <tr> <td> SO_LINGER   </td> <td> Ignored            </td> <td> N/A                   </td> </tr>
 *   <tr> <td> SO_TIMEOUT  </td> <td> Inactivity timeout </td> <td> 4 bytes (milliseconds, MSB first) </td> </tr>
 *   <tr> <td> SO_BINDADDR </td> <td> Local socket IP    </td> <td> 16 bytes              </td> </tr>
 * </table>
 *
 * \param     socket_num  socket to get option information for
 * \param     level       ignored
 * \param     name        option to get
 * \param     buffer      location where option data will be written
 * \param     length      length of the buffer
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #setsockopt
 */
//---------------------------------------------------------------------------
#define getsockopt(socket_num,level,name,buffer,length) syn_getsockopt((socket_num),(name),(buffer))

/**
 * \brief     Set various socket options.
 *
 * Sets a number of supported socket options.  Input data in the
 * buffer depends on the desired socket option.
 *
 * <table>
 *   <tr> <td> <b>Name</b> </td> <td> <b>Description</b> </td> <td> <b>Data in buffer</b> </td> </tr>
 *   <tr> <td> TCP_NODELAY </td> <td> TCP Nagle Enable   </td> <td> 1 byte                </td> </tr>
 *   <tr> <td> SO_LINGER   </td> <td> Ignored            </td> <td> N/A                   </td> </tr>
 *   <tr> <td> SO_TIMEOUT  </td> <td> Inactivity timeout </td> <td> 4 bytes (milliseconds, MSB first) </td> </tr>
 *   <tr> <td> SO_BINDADDR </td> <td> Read only          </td> <td> N/A                   </td> </tr>
 * </table>
 *
 * \param     socket_num  socket to set option information for
 * \param     name        option to set
 * \param     buffer      location of option data that will be written
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #getsockopt
 */
//---------------------------------------------------------------------------
int  syn_setsockopt(int socket_num, unsigned int name, void* buffer);

/**
 * \brief     Set various socket options.
 *
 * Sets a number of supported socket options.  Input data in the
 * buffer depends on the desired socket option.
 *
 * <table>
 *   <tr> <td> <b>Name</b> </td> <td> <b>Description</b> </td> <td> <b>Data in buffer</b> </td> </tr>
 *   <tr> <td> TCP_NODELAY </td> <td> TCP Nagle Enable   </td> <td> 1 byte                </td> </tr>
 *   <tr> <td> SO_LINGER   </td> <td> Ignored            </td> <td> N/A                   </td> </tr>
 *   <tr> <td> SO_TIMEOUT  </td> <td> Inactivity timeout </td> <td> 4 bytes (milliseconds, MSB first) </td> </tr>
 *   <tr> <td> SO_BINDADDR </td> <td> Read only          </td> <td> N/A                   </td> </tr>
 * </table>
 *
 * \param     socket_num  socket to set option information for
 * \param     level       ignored
 * \param     name        option to set
 * \param     buffer      location of option data that will be written
 * \param     length      length of the buffer
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #getsockopt
 */
//---------------------------------------------------------------------------
#define setsockopt(socket_num,level,name,buffer,length) syn_setsockopt((socket_num),(name),(buffer))

/**
 * \brief     Gets the local IP and port of a socket.
 *
 * Stores the local IP and port number of the specified socket in the
 * the <i>address</i> parameter.  Use <i>#getpeername</i> to get
 * the remote port's information for a connection-based (TCP) 
 * socket.
 * 
 * \param     socket_num      handle of the socket to get local IP and port for
 * \param     address         structure where IP and port will be stored
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #getpeername
 */
//---------------------------------------------------------------------------
int  syn_getsockname(int socket_num, struct sockaddr* address);

/**
 * \brief     Gets the local IP and port of a socket.
 *
 * Stores the local IP and port number of the specified socket in the
 * the <i>address</i> parameter.  Use <i>#getpeername</i> to get
 * the remote port's information for a connection-based (TCP) 
 * socket.
 * 
 * \param     socket_num      handle of the socket to get local IP and port for
 * \param     address         structure where IP and port will be stored
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #getpeername
 */
//---------------------------------------------------------------------------
#define getsockname(socket_num,address,address_length)    syn_getsockname((socket_num),(address))

/**
 * \brief     Gets the remote address of a connection-based (TCP socket).
 *
 * Stores the IP address of the remote socket communicating with the socket
 * specified by <i>socket_num</i>.  Use <i>#getsockname</i> to get the local
 * port's information.
 *
 * \param     socket_num      handle of the socket to get remote IP and port for
 * \param     address         structure where IP and port will be stored
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #getsockname
 */
//---------------------------------------------------------------------------
int  syn_getpeername(int socket_num, struct sockaddr* address);

/**
 * \brief     Gets the remote address of a connection-based (TCP socket).
 *
 * Stores the IP address of the remote socket communicating with the socket
 * specified by <i>socket_num</i>.  Use <i>#getsockname</i> to get the local
 * port's information.
 *
 * \param     socket_num      handle of the socket to get remote IP and port for
 * \param     address         structure where IP and port will be stored
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #getsockname
 */
//---------------------------------------------------------------------------
#define getpeername(socket_num,address,address_length)    syn_getpeername((socket_num),(address))


/**
 * \brief     Close all sockets and free the parameter buffer associated with
 *            a task.
 *
 * Close all sockets associated with a process ID and free the parameter buffer.
 *
 * User applications should call this function whenever a task dies or is
 * killed to ensure all associated resources are freed by the socket layer.
 * 
 * \warning   The DS80C400 Silicon Software task scheduler does <b>NOT</b>
 *            call this function.  User applications should call <i>#cleanup</i> 
 *            after each call to <i>#task_kill</i>.
 *
 * \param     process_id   Task PID to clean up sockets associated with, 0 for
 *                         current process.
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #setsockowner
 */
//---------------------------------------------------------------------------
int  cleanup(unsigned int process_id);

/**
 * \brief     Close all sockets and free the parameter buffer associated with
 *            a task.
 *
 * Close all sockets associated with a process ID and free the parameter buffer.
 *
 * User applications should call this function whenever a task dies or is
 * killed to ensure all associated resources are freed by the socket layer.
 * 
 * \warning   The DS80C400 Silicon Software task scheduler does <b>NOT</b>
 *            call this function.  User applications should call <i>#cleanup</i> 
 *            after each call to <i>#task_kill</i>.
 *
 * \param     process_id   Task PID to clean up sockets associated with, 0 for
 *                         current process.
 *
 * \return    0 for success, non-zero for failure.
 */
//---------------------------------------------------------------------------
#define syn_cleanup(process_id)            cleanup((process_id))


/**
 * \brief     Reports number of bytes available on a TCP socket.
 *
 * Reports the number of bytes available on a TCP socket.  This is the number
 * of bytes that can currently be read using the <i>#recv</i> function without
 * blocking.
 *
 * \param     socket_num  the handle of the socket to check for available data
 *
 * \return    The number of bytes available for a <i>#recv</i> function call on this
 *            socket, or -1 on failure.
 *
 * \sa        #recv
 */
//---------------------------------------------------------------------------
int  avail(int socket_num);

/**
 * \brief     Reports number of bytes available on a TCP socket.
 *
 * Reports the number of bytes available on a TCP socket.  This is the number
 * of bytes that can currently be read using the <i>#recv</i> function without
 * blocking.
 *
 * \param     socket_num  the handle of the socket to check for available data
 *
 * \return    The number of bytes available for a <i>#recv</i> function call on this
 *            socket, or 0x0FFFF on failure.
 *
 * \sa        #recv
 */
//---------------------------------------------------------------------------
#define syn_avail(socket_num)              avail((socket_num))

/**
 * \brief     Adds a socket to a specified multicast group.
 *
 * Adds a UDP socket to a specified multicast group.  In order to receive
 * multicasts from a group, first <i>#bind</i> the socket to the port number
 * that the multicast group is using (it is not sufficient to include it
 * here in order to receive).
 *
 * Use the <i>#leave</i> function to leave a multicast group.
 *
 * \warning   IPv6 multicasting is not supported
 *
 * \param     socket_num      handle for the datagram socket that will join a multicast group
 * \param     address         IP address of the multicast group to join
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #leave
 */
//---------------------------------------------------------------------------
int  syn_join(int socket_num, struct sockaddr* address);

/**
 * \brief     Adds a socket to a specified multicast group.
 *
 * Adds a UDP socket to a specified multicast group.  In order to receive
 * multicasts from a group, first <i>#bind</i> the socket to the port number
 * that the multicast group is using (it is not sufficient to include it
 * here in order to receive).
 *
 * Use the <i>#leave</i> function to leave a multicast group.
 *
 * \warning   IPv6 multicasting is not supported
 *
 * \param     socket_num      handle for the datagram socket that will join a multicast group
 * \param     address         IP address of the multicast group to join
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #leave
 */
//---------------------------------------------------------------------------
#define join(socket_num,address,address_length)           syn_join((socket_num),(address))

/**
 * \brief     Removes a socket from the specified multicast group.
 *
 * Removes a UDP socket from the specified multicast group.
 *
 * \param     socket_num      handle for the datagram socket that will leave a multicast group
 * \param     address         IP address of the multicast group to leave
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #join
 */
//---------------------------------------------------------------------------
int  syn_leave(int socket_num, struct sockaddr* address);

/**
 * \brief     Removes a socket from the specified multicast group.
 *
 * Removes a UDP socket from the specified multicast group.
 *
 * \param     socket_num      handle for the datagram socket that will leave a multicast group
 * \param     address         IP address of the multicast group to leave
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #join
 */
//---------------------------------------------------------------------------
#define leave(socket_num,address,address_length)          syn_leave((socket_num),(address))


/**
 * \brief     Get the IPv4 configuration parameters.
 *
 * Get the IPv4 configuration parameters, including IP address, subnet mask, 
 * and gateway.  The parameters are returned in a buffer in the following
 * form:
 * <table>
 *   <tr> <td> <b>Parameter</b> </td> <td> <b>Offset</b> </td> <td> <b>Length</b> </td> <td> <b>Description</b> </td> </tr>
 *   <tr> <td> (zero)           </td> <td> 0             </td> <td> 12            </td> <td> Must be 0          </td> </tr>
 *   <tr> <td> IP4ADDR          </td> <td> 12            </td> <td> 4             </td> <td> IP address         </td> </tr>
 *   <tr> <td> IP4SUBNET        </td> <td> 16            </td> <td> 4             </td> <td> Subnet mask        </td> </tr>
 *   <tr> <td> IP4PREFIX        </td> <td> 20            </td> <td> 1             </td> <td> Number of 1 bits in subnet mask </td> </tr>
 *   <tr> <td> (zero)           </td> <td> 21            </td> <td> 12            </td> <td> Must be 0          </td> </tr>
 *   <tr> <td> IP4GATEWAY       </td> <td> 33            </td> <td> 4             </td> <td> Gateway IP address </td> </tr>
 * </table>
 *
 * IPv6 addresses are autoconfigured.  To retrieve the IPv6 address, use the 
 * <i>#getipv6params</i> function.
 *
 * \param     param_buffer  pointer to buffer to store IP configuration data
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #setnetworkparams
 * \sa        #getipv6params
 */
//---------------------------------------------------------------------------
int  getnetworkparams(void* param_buffer);

/**
 * \brief     Get the IPv4 configuration parameters.
 *
 * Get the IPv4 configuration parameters, including IP address, subnet mask, 
 * and gateway.  The parameters are returned in a buffer in the following
 * form:
 * <table>
 *   <tr> <td> <b>Parameter</b> </td> <td> <b>Offset</b> </td> <td> <b>Length</b> </td> <td> <b>Description</b> </td> </tr>
 *   <tr> <td> (zero)           </td> <td> 0             </td> <td> 12            </td> <td> Must be 0          </td> </tr>
 *   <tr> <td> IP4ADDR          </td> <td> 12            </td> <td> 4             </td> <td> IP address         </td> </tr>
 *   <tr> <td> IP4SUBNET        </td> <td> 16            </td> <td> 4             </td> <td> Subnet mask        </td> </tr>
 *   <tr> <td> IP4PREFIX        </td> <td> 20            </td> <td> 1             </td> <td> Number of 1 bits in subnet mask </td> </tr>
 *   <tr> <td> (zero)           </td> <td> 21            </td> <td> 12            </td> <td> Must be 0          </td> </tr>
 *   <tr> <td> IP4GATEWAY       </td> <td> 33            </td> <td> 4             </td> <td> Gateway IP address </td> </tr>
 * </table>
 *
 * IPv6 addresses are autoconfigured.  To retrieve the IPv6 address, use the 
 * <i>#getipv6params</i> function.
 *
 * \param     param_buffer  pointer to buffer to store IP configuration data
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #setnetworkparams
 * \sa        #getipv6params
 */
//---------------------------------------------------------------------------
#define syn_getnetworkparams(param_buffer)   getnetworkparams((param_buffer))

/**
 * \brief     Set the IPv4 configuration parameters.
 *
 * Set the IPv4 configuration parameters, including IP address, subnet mask, 
 * and gateway.  Input parameters should be formatted in the following
 * form:
 * <table>
 *   <tr> <td> <b>Parameter</b> </td> <td> <b>Offset</b> </td> <td> <b>Length</b> </td> <td> <b>Description</b> </td> </tr>
 *   <tr> <td> (zero)           </td> <td> 0             </td> <td> 12            </td> <td> Must be 0          </td> </tr>
 *   <tr> <td> IP4ADDR          </td> <td> 12            </td> <td> 4             </td> <td> IP address         </td> </tr>
 *   <tr> <td> IP4SUBNET        </td> <td> 16            </td> <td> 4             </td> <td> Subnet mask        </td> </tr>
 *   <tr> <td> IP4PREFIX        </td> <td> 20            </td> <td> 1             </td> <td> Number of 1 bits in subnet mask </td> </tr>
 *   <tr> <td> (zero)           </td> <td> 21            </td> <td> 12            </td> <td> Must be 0          </td> </tr>
 *   <tr> <td> IP4GATEWAY       </td> <td> 33            </td> <td> 4             </td> <td> Gateway IP address </td> </tr>
 * </table>
 *
 * Use this method to give the DS80C400 a static IP address.  To dynamically
 * configure an IP address, use methods from the DHCP library in rom400_dhcp.h
 * (IP addresses leased by the DHCP client can still be retrieved
 * by calling <i>#getnetworkparams</i>).
 * 
 * IPv6 addresses are autoconfigured.  To retrieve the IPv6 address, use the 
 * <i>#getipv6params</i> function.
 * 
 * \param     param_buffer  pointer to buffer with IP configuration data
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #getnetworkparams
 * \sa        #getipv6params
 */
//---------------------------------------------------------------------------
int  setnetworkparams(void* param_buffer);

/**
 * \brief     Set the IPv4 configuration parameters.
 *
 * Set the IPv4 configuration parameters, including IP address, subnet mask, 
 * and gateway.  Input parameters should be formatted in the following
 * form:
 * <table>
 *   <tr> <td> <b>Parameter</b> </td> <td> <b>Offset</b> </td> <td> <b>Length</b> </td> <td> <b>Description</b> </td> </tr>
 *   <tr> <td> (zero)           </td> <td> 0             </td> <td> 12            </td> <td> Must be 0          </td> </tr>
 *   <tr> <td> IP4ADDR          </td> <td> 12            </td> <td> 4             </td> <td> IP address         </td> </tr>
 *   <tr> <td> IP4SUBNET        </td> <td> 16            </td> <td> 4             </td> <td> Subnet mask        </td> </tr>
 *   <tr> <td> IP4PREFIX        </td> <td> 20            </td> <td> 1             </td> <td> Number of 1 bits in subnet mask </td> </tr>
 *   <tr> <td> (zero)           </td> <td> 21            </td> <td> 12            </td> <td> Must be 0          </td> </tr>
 *   <tr> <td> IP4GATEWAY       </td> <td> 33            </td> <td> 4             </td> <td> Gateway IP address </td> </tr>
 * </table>
 *
 * Use this method to give the DS80C400 a static IP address.  To dynamically
 * configure an IP address, use methods from the DHCP library in rom400_dhcp.h
 * (IP addresses leased by the DHCP client can still be retrieved
 * by calling <i>#getnetworkparams</i>).
 * 
 * IPv6 addresses are autoconfigured.  To retrieve the IPv6 address, use the 
 * <i>#getipv6params</i> function.
 * 
 * \param     param_buffer  pointer to buffer with IP configuration data
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #getnetworkparams
 * \sa        #getipv6params
 */
//---------------------------------------------------------------------------
#define syn_setnetworkparams(param_buffer)   setnetworkparams((param_buffer))

/**
 * \brief     Get the IPv6 address.
 *
 * Gets the IPv6 address of the ethernet interface.  The format for
 * the buffer after this function returns is:
 * <table>
 *   <tr> <td> <b>Parameter</b> </td> <td> <b>Offset</b> </td> <td> <b>Length</b> </td> <td> <b>Description</b> </td> </tr>
 *   <tr> <td> IP6ADDR          </td> <td> 0             </td> <td> 16            </td> <td> IP address         </td> </tr>
 *   <tr> <td> IP6PREFIX        </td> <td> 16            </td> <td> 1             </td> <td> IP prefix length   </td> </tr>
 * </table>
 * 
 * \param     param_buffer  pointer to buffer to store IPv6 configuration data
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #getnetworkparams
 * \sa        #setnetworkparams
 */
//---------------------------------------------------------------------------
int  getipv6params(void* param_buffer);

/**
 * \brief     Get the IPv6 address.
 *
 * Gets the IPv6 address of the ethernet interface.  The format for
 * the buffer after this function returns is:
 * <table>
 *   <tr> <td> <b>Parameter</b> </td> <td> <b>Offset</b> </td> <td> <b>Length</b> </td> <td> <b>Description</b> </td> </tr>
 *   <tr> <td> IP6ADDR          </td> <td> 0             </td> <td> 16            </td> <td> IP address         </td> </tr>
 *   <tr> <td> IP6PREFIX        </td> <td> 16            </td> <td> 1             </td> <td> IP prefix length   </td> </tr>
 * </table>
 * 
 * \param     param_buffer  pointer to buffer to store IPv6 configuration data
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #getnetworkparams
 * \sa        #setnetworkparams
 */
//---------------------------------------------------------------------------
#define syn_getipv6params(param_buffer)      getipv6params((param_buffer))

/**
 * \brief     Get the ethernet status.
 *
 * Returns the ethernet status byte.  This is a bit-wise OR of the following
 * flags:
 * <table>
 *   <tr> <td> <b>Flag</b>      </td> <td> <b>Value</b>  </td> <td> <b>Description</b>    </td> </tr>
 *   <tr> <td> ETH_STATUS_LINK  </td> <td> 01h           </td> <td> Ethernet link status  </td> </tr>
 * </table>
 *  
 * Currently, no other flags are defined.
 *
 * \return    Bitmapped ethernet status byte.
 */
//---------------------------------------------------------------------------
unsigned int  getethernetstatus(void);

/**
 * \brief     Get the ethernet status.
 *
 * Returns the ethernet status byte.  This is a bit-wise OR of the following
 * flags:
 * <table>
 *   <tr> <td> <b>Flag</b>      </td> <td> <b>Value</b>  </td> <td> <b>Description</b>    </td> </tr>
 *   <tr> <td> ETH_STATUS_LINK  </td> <td> 01h           </td> <td> Ethernet link status  </td> </tr>
 * </table>
 *  
 * Currently, no other flags are defined.
 *
 * \return    Bitmapped ethernet status byte.
 */
//---------------------------------------------------------------------------
#define syn_getethernetstatus()   getethernetstatus()

/**
 * \brief     Get the address of the TFTP server.
 *
 * Returns the address of the server accessed by the TFTP functions.  To 
 * communicate with a TFTP server, use the functions listed in rom400_tftp.h,
 * the TFTP library.
 *
 * \param     address         structure to store the address of the TFTP server
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #settftpserver
 */
//---------------------------------------------------------------------------
int  syn_gettftpserver(struct sockaddr* address);

/**
 * \brief     Get the address of the TFTP server.
 *
 * Returns the address of the server accessed by the TFTP functions.  To 
 * communicate with a TFTP server, use the functions listed in rom400_tftp.h,
 * the TFTP library.
 *
 * \param     address         structure to store the address of the TFTP server
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #settftpserver
 */
//---------------------------------------------------------------------------
#define gettftpserver(address,address_length)    syn_gettftpserver((address))

/**
 * \brief     Set the address of the TFTP server.
 *
 * Set the address of the server that the TFTP functions will use.  
 * The <i>#settftpserver</i> function must be used if the address of 
 * the TFTP server is not acquired by DHCP or 1-Wire.  Once the TFTP
 * server's address is set, use the functions listed in rom400_tftp.h
 * to begin receiving files.
 *
 * \param     address         structure to store the address of the TFTP server
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #gettftpserver
 */
//---------------------------------------------------------------------------
int  syn_settftpserver(struct sockaddr* address);

/**
 * \brief     Set the address of the TFTP server.
 *
 * Set the address of the server that the TFTP functions will use.  
 * The <i>#settftpserver</i> function must be used if the address of 
 * the TFTP server is not acquired by DHCP or 1-Wire.  Once the TFTP
 * server's address is set, use the functions listed in rom400_tftp.h
 * to begin receiving files.
 *
 * \param     address         structure to store the address of the TFTP server
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure
 *
 * \sa        #gettftpserver
 */
//---------------------------------------------------------------------------
#define settftpserver(address,address_length)    syn_settftpserver((address))


/**
 * \brief     Clears the parameter buffers used by the socket library.
 *
 * Clears buffers used to store parameters for the socket library.  This
 * function should be called immediately after calling the <i>#init_rom</i>
 * function, and before any socket library functions are called.
 *
 * \sa        #init_rom
 */
//---------------------------------------------------------------------------
void         clear_param_buffers(void);


/**
 * \brief     Returns the version number of this socket library.
 *
 * \return    Version number of this SOCK library.
 */
//---------------------------------------------------------------------------
unsigned int sock_version(void);

/**
 * \brief     Returns the version number of this socket library.
 *
 * \return    Version number of this SOCK library.
 */
#define syn_version()        sock_version()


/**
 * \brief     Generate an ARP request for a given IPv4 address.
 *
 * Unconditionally generate an ARP request for a given IPv4 address.
 * This functionality can be used to implement Zeroconf protocols.
 *
 * \param     address         structure to store the address
 *
 * \return    0 for success, non-zero for failure
 */
//---------------------------------------------------------------------------
int  syn_arp_generaterequest(struct sockaddr* address);

/**
 * \brief     Unconditionally generate an ARP request for a given IPv4 address.
 *
 * Unconditionally generate an ARP request for a given IPv4 address.
 * This functionality can be used to implement Zeroconf protocols.
 *
 * \param     address         structure to store the address
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure
 */
//---------------------------------------------------------------------------
#define arp_generaterequest(address,address_length)    syn_arp_generaterequest((address))

/**
 * \brief     Generate an ARP request for a given IPv4 address and add to the ARP cache.
 *
 * If the given IP address is not in the ARP cache, generate an ARP request and
 * add it to the cache.
 *
 * \param     address         structure to store the address
 *
 * \return    0 for success, non-zero for failure
 */
//---------------------------------------------------------------------------
int  syn_arp_cacherequest(struct sockaddr* address);

/**
 * \brief     Generate an ARP request for a given IPv4 address and add to the ARP cache.
 *
 * If the given IP address is not in the ARP cache, generate an ARP request and
 * add it to the cache.
 *
 * \param     address         structure to store the address
 * \param     address_length  the length of the address structure (ignored)
 *
 * \return    0 for success, non-zero for failure
 */
//---------------------------------------------------------------------------
#define arp_cacherequest(address,address_length)    syn_arp_cacherequest((address))


/**
 * \brief     Returns the number of sockets in the wait queue for this 
 *            listening socket.
 *
 * Returns the number of sockets in the queue attempting to connect to this
 * server socket.
 *
 * \param     socket_handle   handle to socket to check for waiting 
 *                            connections
 * \param     address         location where the IP and port number will be 
 *                            written
 *
 * \return    -1 if the socket is not a streaming socket set up to listen
 *            0 or greater for the number of sockets waiting
 *
 *            The IP and port of the socket are returned in <i>address</i>.
 */
//---------------------------------------------------------------------------
int  acceptqueue(int socket_handle, struct sockaddr* address);

/**
 * \brief     Returns whether or not data is available to be read on a
 *            datagram socket.
 *
 * Returns <b>1</b> if there is data available to be read on a UDP socket.
 *
 * \param     socket_handle   handle to socket to check for available 
 *                            datagrams
 * \param     address         location where the IP and port number will be 
 *                            written
 *
 * \return    -1 if the socket is not a datagram socket 
 *            0 if no datagram packets are available
 *            1 if a datagram is available
 * 
 *            The IP and port of the socket are returned in <i>address</i>.
 */
//---------------------------------------------------------------------------
int  udpavailable(int socket_handle, struct sockaddr* address);


/**
 * \brief     Closes a specific socket.
 *
 * Closes the specified socket that was created using the <i>socket</i> 
 * function. 
 *
 * \param     socket_num  the socket handle to close
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #socket
 */
//---------------------------------------------------------------------------
int closesocket(int socket_num);

/**
 * \brief     Closes a specific socket.
 *
 * Closes the specified socket that was created using the <i>socket</i> 
 * function. 
 *
 * \param     socket_num  the socket handle to close
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #socket
 */
//---------------------------------------------------------------------------
#define syn_closesocket(socket_num)        closesocket((socket_num))


/**
 * \brief     Get the pointer to the MAC ID storage area.
 * 
 * Returns the pointer to the MAC ID storage area.  This area will
 * store the MAC ID after a successful call to <i>#setmacid</i>.
 *
 * \return    Pointer to the 400's MAC ID (6 bytes stored at this location)
 *
 * \sa        #setmacid
 */
//---------------------------------------------------------------------------
unsigned char* getmacid(void);

/**
 * \brief     Get the pointer to the MAC ID storage area.
 * 
 * Returns the pointer to the MAC ID storage area.  This area will
 * store the MAC ID after a successful call to <i>#setmacid</i>.
 *
 * \return    Pointer to the 400's MAC ID (6 bytes stored at this location)
 *
 * \sa        #setmacid
 */
//---------------------------------------------------------------------------
#define syn_getmacid()            getmacid()

/**
 * \brief     Stores the MAC ID into the MAC ID storage area.
 *
 * This is a redirected function.  The DS80C400's default implementation
 * of this function searches the 1-Wire for a DS2502U-E48 1-Wire chip 
 * which contains a MAC ID.  This MAC ID is then stored into the MAC ID 
 * storage area, the location of which is stored in a pointer in the 
 * export table.  Use the <i>#getmacid</i> function to return a pointer 
 * to the MAC ID storage area.
 *
 * \sa        #getmacid
 */
//---------------------------------------------------------------------------
void          setmacid(void);

/**
 * \brief     Stores the MAC ID into the MAC ID storage area.
 *
 * This is a redirected function.  The DS80C400's default implementation
 * of this function searches the 1-Wire for a DS2502U-E48 1-Wire chip 
 * which contains a MAC ID.  This MAC ID is then stored into the MAC ID 
 * storage area, the location of which is stored in a pointer in the 
 * export table.  Use the <i>#getmacid</i> function to return a pointer 
 * to the MAC ID storage area.
 *
 * \sa        #getmacid
 */
//---------------------------------------------------------------------------
#define syn_setmacid()            setmacid()


/**
 * \brief     Pings the specified address.
 *
 * Sends an ICMP echo request (ping) to a specified address.  Note that this function
 * is <b>NOT</b> safe to be called from multiple processes at the same time.
 *
 * \param     address         IP address to send an ICMP echo request to
 * \param     address_length  the length of the address structure (ignored)
 * \param     time_to_live    packets send by ping have this "time to live" setting
 * \param     response        data structure to fill in returned data (this argument must not be NULL)
 *
 * \return    response time in milliseconds (0 means less than 1ms), or -1L for failure
 *
 * The ping return data structure is defined as follows:
 *   reserved - Reserved field
 *   ip_header - The IP header of the return packet
 *   icmp_header - The ICMP header of the return packet
 *   icmp_data - The ICMP data portion of the return packet (should be 0x20,0x21,0x22,...,0x3f)
 */
//---------------------------------------------------------------------------
long ping(struct sockaddr* address, unsigned int address_length, unsigned int time_to_live, struct pingdata* response);


/**
 * \brief     Read a PHY register via MII.
 *
 * This function reads a PHY register via the MII interface. See the IEEE 802.3
 * specification (22.2.4) for a description of the MII management register set.
 *
 * \param     phy             PHY address (0 to 31)
 * \param     reg	      register address (0 to 31, 16 through 31 are vendor
 *                            specific)
 *
 * \return    value read from the register specified
 */
//---------------------------------------------------------------------------
unsigned int eth_readmii(unsigned int phy, unsigned int reg);

/**
 * \brief     Write a PHY register via MII.
 *
 * This function writes a PHY register via the MII interface. See the IEEE 802.3
 * specification (22.2.4) for a description of the MII management register set.
 *
 * \param     phy             PHY address (0 to 31)
 * \param     reg	      register address (0 to 31, 16 through 31 are vendor
 *                            specific)
 * \param     val             value to write to the specified register
 */
//---------------------------------------------------------------------------
void eth_writemii(unsigned int phy, unsigned int reg, unsigned int val);


/**
 * \brief     Disable multicast hardware receiver.
 *
 * This function disables the "pass multicast" (PM) bit in the DS80C400 MAC control
 * register. This improves performance if the application doesn't use multicast.
 * This function must be called after initialization of the Ethernet.
 * WARNING: IPv6 requires multicast. Disabling the receiver disables IPv6
 * address resolution.
 */
//---------------------------------------------------------------------------
void eth_disablemulticastreceiver(void);

/**
 * \brief     Unbind a bound socket.
 *
 * Removes a local address and port from a socket that was assigned to it using
 * <i>#bind</i>.
 *
 * \param     socket_num      socket handle
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #bind
 */
//---------------------------------------------------------------------------
int  unbind(int socket_num);

/**
 * \brief     Sets the socket's owner to a different task ID.
 *
 * Sets the socket owner to a different task ID. This is useful where program
 * code relies on <i>#cleanup</i> to deallocate a process' resources, or in
 * cases where ownership of a socket needs to be moved to a child process.
 * Note that the new process ID is not checked for validity and it is possible
 * to assign a socket to a non-existent task.
 *
 * \param     socket_num      socket handle
 * \param     process_id      the task PID of the new socket owner
 *
 * \return    0 for success, non-zero for failure.
 *
 * \sa        #cleanup
 */
//---------------------------------------------------------------------------
int  setsockowner(int socket_num, unsigned int process_id);


/**
 * \brief     Read a MAC CSR register.
 *
 * This function reads a MAC CSR register from the DS80C400. See the data sheet
 * and user's guide for more information about the CSR registers.
 *
 * \param     reg	      register address (0 to 0x2c in steaps of 4)
 *
 * \return    value read from the register specified
 *
 * \sa        #eth_writecsr
 */
//---------------------------------------------------------------------------
unsigned long eth_readcsr(unsigned int reg);

/**
 * \brief     Write a MAC CSR register.
 *
 * This function writes a MAC CSR register. See the DS80C400 data sheet
 * and user's guide for more information about the CSR registers.
 *
 * \param     reg	      register address (0 to 0x2c in steaps of 4)
 * \param     val             value to write to the specified register
 *
 * \sa        #eth_readcsr
 */
//---------------------------------------------------------------------------
void eth_writecsr(unsigned int reg, unsigned long val);

#endif

