/*---------------------------------------------------------------------------
 *  Copyright (C) 2003-2006 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 *
 * This file contains function definitions for the built-in ROM functions 
 * of the Dallas Semiconductor 400 processor.  This file is intended for use 
 * with the Keil MicroVision (uVision) C compiler.
 *
 * ---------------------------------------------------------------------------
 */
#ifndef __rom400_task_
#define __rom400_task_

#pragma SAVE
#pragma REGPARMS

/** \file rom400_task.h
 *  \brief Process scheduler functions in the DS80C400 ROM
 *
 *  This library contains functions for starting, suspending, 
 *  killing, and managing tasks using the ROM's process scheduler.
 *  
 *  For detailed information on the DS80C400 please see the
 *  <a href="http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf">
 *  High-Speed Microcontroller User's Guide: DS80C400 Supplement</a>.
 *
 *  \warning  Some functions in this library are <b>NOT</b> multi-process 
 *            safe--that is, if you call the same method from two different 
 *            processes at the same time, the parameters to the function 
 *            may be destroyed, yielding unpredictable results.  Consult
 *            each individual funtion's documentation for details on which
 *            functions are multi-process safe.
 */

/** Version number associated with this header file.  Should be the same as
 * the version number returned by the <i>#task_version</i> function.
 * \sa #task_version */
#define ROM400_TASK_VERSION         9

/// Included for legacy reasons.  Please use #ROM400_TASK_VERSION instead.
#define ROM400_SCHED_VERSION         ROM400_TASK_VERSION

/** Timer reload value for 14.746 MHz crystal.
 * \sa #task_settickreload
 * \sa #task_gettickreload */
#define RELOAD_14_746 0xfb33

/** Timer reload value for 18.432 MHz crystal.
 * \sa #task_settickreload
 * \sa #task_gettickreload */
#define RELOAD_18_432 0xfa00

/** Timer reload value for 29.491 MHz crystal.
 * \sa #task_settickreload
 * \sa #task_gettickreload */
#define RELOAD_29_491 0xfd99

/** Timer reload value for 36.864 MHz crystal.
 * \sa #task_settickreload
 * \sa #task_gettickreload */
#define RELOAD_36_864 0xfd00

/** Timer reload value for 58.982 MHz crystal.
 * \sa #task_settickreload
 * \sa #task_gettickreload */
#define RELOAD_58_982 0xfecc

/** Timer reload value for 73.728 MHz crystal.
 * \sa #task_settickreload
 * \sa #task_gettickreload */
#define RELOAD_73_728 0xfe80


/** Minimum priority level assignable to a task.
 * \sa #task_setpriority
 * \sa #task_getpriority */
#define MIN_PRIORITY    1
/** Normal priority for a task.  This is the default priority for the default task.
 * \sa #task_setpriority
 * \sa #task_getpriority */
#define NORM_PRIORITY   128
/** Maximum priority level assignable to a task.
 * \sa #task_setpriority
 * \sa #task_getpriority */
#define MAX_PRIORITY    255

/** Event flag for putting a task to sleep.
 * \sa #task_wait */
#define FLAG_SLEEPING   1
/** Event flag for putting a task to sleep. Reserved by the system.
 * \sa #task_wait */
#define FLAG_IO_WAIT    2
/** Event flag for putting a task to sleep. Reserved by the system.
 * \sa #task_wait */
#define FLAG_DHCP_WAIT  4
/** Event flag for putting a task to sleep.
 * \sa #task_wait */
#define FLAG_USER0      8
/** Event flag for putting a task to sleep.
 * \sa #task_wait */
#define FLAG_USER1      0x10
/** Event flag for putting a task to sleep.
 * \sa #task_wait */
#define FLAG_USER2      0x20
/** Event flag for putting a task to sleep.
 * \sa #task_wait */
#define FLAG_USER3      0x40
/** Event flag for putting a task to sleep.
 * \sa #task_wait */
#define FLAG_USER4      0x80

/** Default size for task switching buffer.
 * \sa #task_genesis */
#define ROM_SAVESIZE    384

/** Structure to be used when handling the DS80C400's 5 byte time values.
 * \sa #task_gettimemillis
 */
struct TIME
{
    /// Most significant byte of the time stamp.  The Keil compiler does not have data types longer than 4 bytes.
    unsigned char msb;
    /// The lower 4 bytes of a DS80C400 time stamp (in milliseconds).  This will cover up to 49.7 days.
    unsigned long millis;
};


/** Structure that defines a raw 24-bit memory pointer. Unlike void far*, this pointer cannot be
 *  directly used in Keil C. To convert it into a far pointer, increase the highest byte by 1 and
 *  set the top bit depending on the memory space.
 */
struct FARPTR
{
    /// Most significant (raw) byte of the memory address
    unsigned char  msb;
    /// Offset within the 64KB segment of msb.
    unsigned short offset;
};

/**
 * Task control buffer.
 */
struct TCB
{
    unsigned char  Priority;     ///< Priority of the task
    unsigned char  ID;           ///< ID of the task
    struct FARPTR  Next;         ///< Next task in the queue
    unsigned short MemHandle;    ///< KMalloc handle for this TCB
    unsigned char  Flags;        ///< Flags for the task
    struct TIME    WakeupTime;   ///< Time that the task is scheduled to wake from a sleep
    unsigned short StateSize;    ///< Size of the saved state for the task
    struct FARPTR  StatePtr;     ///< Pointer to the saved state for the task
};

/**
 * \brief     Initializes the process scheduler.
 *
 * Note that calling the function <i>#init_rom</i> from the initialization
 * library is the preferred way of initializing the ROM.
 *
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \param     savesize Size of the task buffer for saving information on task switches.
 */
//---------------------------------------------------------------------------
void           task_genesis(unsigned int savesize);

/**
 * \brief     Gets the process ID for the current task.
 *
 * Returns the process ID for the current task, which can be used to manage that
 * task.
 *
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \return    PID for the current task.
 *
 * \sa        #task_kill
 * \sa        #task_setpriority
 * \sa        #task_getpriority
 */
//---------------------------------------------------------------------------
unsigned char  task_getcurrent(void);

/**
 * \brief     Gets the priority level for the given task.
 *
 * Given the process ID of a task, return the priority level
 * for that task.  Use a <i>task_id</i> of 0 for the current 
 * task.
 *
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \param     task_id Task PID to get the priority for.  A task PID of zero 
 *                    means the current task.
 *
 * \return    Priority level of the task.
 *
 * \sa        #MIN_PRIORITY
 * \sa        #NORM_PRIORITY
 * \sa        #MAX_PRIORITY
 */
//---------------------------------------------------------------------------
unsigned char  task_getpriority(unsigned char task_id);

/**
 * \brief     Sets the priority level for a given task.
 *
 * Given the process ID of a task, set the priority level
 * for that task.  Use a <i>task_id</i> of 0 for the current
 * task.
 *
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \param     task_id    Task PID to set the priority for.  A task PID of zero 
 *                       means the current task.
 * \param     priority   Priority setting for PID <i>task_id</i>.  Can be any 
 *                       value between #MIN_PRIORITY and #MAX_PRIORITY
 *
 * \return    0 for Success, non-zero for failure
 *
 * \sa        #MIN_PRIORITY
 * \sa        #NORM_PRIORITY
 * \sa        #MAX_PRIORITY
 */
//---------------------------------------------------------------------------
unsigned char  task_setpriority(unsigned char task_id, unsigned char priority);

/**
 * \brief     Creates a new task.
 *
 * Spawns a new task, returning the process ID of the new task to the parent
 * task.  Note that because of the way the Keil compiler assigns variables,
 * calls to task_fork should be wrapped inside a critical section.  Make sure 
 * the child's process ID is stored in a secure location before exiting the 
 * critical section.  Note that only the parent need leave the critical 
 * section, the child will not run until the parent has left it.
 *
 * This function is safe to be called from multiple processes at the same 
 * time.
 * 
 * \param     priority  priority level for the new task.
 * \param     savesize  size of the task state buffer for the new task
 *
 * \return    0x0FFFF for failure, else 0 if this is the child task, or the 
 *            child's PID if this is the parent.
 *
 * \sa        #MIN_PRIORITY
 * \sa        #NORM_PRIORITY
 * \sa        #MAX_PRIORITY
 * \sa        #ROM_SAVESIZE
 * \sa        #task_kill
 * \sa        #task_reentrant_stack
 */
//---------------------------------------------------------------------------
unsigned int   task_fork(unsigned char priority, unsigned int savesize);

/**
 * \brief     Kills the specified task.
 *
 * Kill the specified task.  Use a <i>task_id</i> of 0 to indicate the current
 * task.  This function does not close or clean up any sockets.  Use the socket
 * library function <i>#cleanup</i> to clean any sockets owned by the task
 * before any more processes are created.
 * 
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \param     task_id  Task PID to kill.
 *
 * \return    0 for Success, non-zero for failure
 *
 * \sa        #task_fork
 */
//---------------------------------------------------------------------------
unsigned char  task_kill(unsigned char task_id);

/**
 * \brief     Suspends the specified task.
 *
 * Suspends the execution of the specified task until all specified events
 * have occurred.  Use the function <i>#task_signal</i> to wake the task
 * up.
 * Use the event flags <i>FLAG_USER0</i> through <i>FLAG_USER4</i> or a
 * bitwise OR of these flags only, all other bits are system reserved.
 * 
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \param     task_id     Task PID to suspend.  A task PID of zero 
 *                        means suspend the current task.
 * \param     event_mask  Bitmap of events to wait for before wakeup.
 *
 * \return    0 for Success, non-zero for failure
 *
 * \sa        #task_signal
 * \sa        #task_sleep
 */
//---------------------------------------------------------------------------
unsigned char  task_suspend(unsigned char task_id, unsigned char event_mask);

////// Note: task_synch_wait is deprecated use task_wait
/// @cond
unsigned char  task_synch_wait(unsigned int millis_h, unsigned int millis_l, unsigned int task_event);
/// @endcond

/**
 * \brief     Puts the specified task to sleep.
 *
 * Suspends the execution of the specified task until all specified events
 * have occurred, and/or until a set amount of time has elapsed.
 *
 * When calling <i>task_wait</i>, use the event flags <i>FLAG_USER0</i> through
 * <i>FLAG_USER4</i> or a bitwise OR of these flags only, all other bits are system
 * reserved.
 *
 * Use the function <i>#task_signal</i> to wake the task up. To properly wake
 * a task, specify <i>FLAG_SLEEPING</i> in the call to <i>task_signal</i> as
 * well as the user event flag(s).
 *
 * This function is safe to be called from multiple processes at the same
 * time.
 *
 * \param     task_id     Task PID to put to sleep.  A task PID of zero
 *                        means put the current task to sleep.
 * \param     event_mask  Bitmap of events to wait for before wakeup.
 * \param     millis      Maximum number of milliseconds to sleep for.
 *
 * \return    0 for Success, non-zero for failure
 *
 * \sa        #task_signal
 * \sa        #task_sleep
 * \sa        #task_suspend
 */
//---------------------------------------------------------------------------
unsigned char task_wait(unsigned char task_id, unsigned char event_mask, long millis);

/**
 * \brief     Posts events to the specified task.
 *
 * Sends the event(s) in <i>event_mask</i> to process <i>task_id</i>.  If the
 * task is waiting for no other events, it will wake up and be
 * electable to run by the task scheduler. 
 *
 * Use the event flags <i>FLAG_USER0</i> through <i>FLAG_USER4</i> or a
 * bitwise OR of these flags. For tasks suspended in <i>task_wait</i>,
 * <i>FLAG_SLEEPING</i> should also be specified, otherwise the task will
 * sleep until the sleep time has elapsed.
 * <i>FLAG_SLEEPING</i> can also be posted to prematurely wake a task
 * suspended in <i>task_sleep</i>.
 *
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \param     task_id     Task PID to signal.
 * \param     event_mask  Bitmap of events to signal.
 *
 * \return    0 for Success, non-zero for failure
 * 
 * \sa        #task_sleep
 * \sa        #task_suspend
 * \sa        #task_wait
 */
//---------------------------------------------------------------------------
unsigned char  task_signal(unsigned char task_id, unsigned char event_mask);

/**
 * \brief     Returns the system tick count.
 *
 * The default implementation of this function returns the approximate 
 * number of milliseconds since the system started.  Note that the largest
 * raw data structure supported by Keil is 4 bytes, yet the DS80C400's 
 * tick counter is 5 bytes, therefore the special <i>#TIME</i> structure
 * is used.
 *
 * This is a redirected function.  The ROM includes a default 
 * process scheduler implementation.  See the 
 * <a href="http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf">
 * DS80C400 User's Guide</a> for information on replacing the default process scheduler 
 * with your own.
 * 
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \param     t  pointer to a structure of type #TIME (a 5-byte structure).  The 
 *               result is written to this pointer, MSB first.
 * 
 * \sa        #TIME
 */
//---------------------------------------------------------------------------
void           task_gettimemillis(struct TIME* t);

/**
 * \brief     Redirected function to return the current thread's ID number.
 *
 * This is a redirected function that should be used to retrieve the current
 * thread's ID number.  However, the DS80C400 ROM does not support threads,
 * so the default implementation of this function always returns 0x01.
 *
 * For more information on redirected functions, see the section <i>ROM Redirect 
 * Function Table</i> in the DS80C400 User's Guide at 
 * http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf.
 * 
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \return    default implementation returns 0x01
 */
//---------------------------------------------------------------------------
unsigned char  task_getthreadid();

/**
 * \brief     Redirected function to resume the specified thread.
 *
 * This is a redirected function that should be used to resume a suspended 
 * or sleeping thread.  However, the DS80C400 ROM does not support
 * threads, so the default implementation of this function resumes the
 * task with a process ID matching <i>task</i>.
 *
 * For more information on redirected functions, see the section <i>ROM Redirect 
 * Function Table</i> in the DS80C400 User's Guide at 
 * http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf
 * 
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \param     thread  thread ID to resume
 * \param     task    ID of the process that <i>thread</i> belongs to
 *
 * \return    0 for Success, non-zero for failure
 * 
 * \sa        #task_threadiosleep
 * \sa        #task_threadiosleepnc
 */
//---------------------------------------------------------------------------
unsigned char  task_threadresume(unsigned char thread, unsigned char task);

/**
 * \brief     Redirected function to put the current thread to sleep.
 *
 * This is a redirected function that should be used to put a thread to 
 * sleep.  However, the DS80C400 does not support threads, so the default
 * implementation of this function puts the current task to sleep.
 *
 * For more information on redirected functions, see the section <i>ROM Redirect 
 * Function Table</i> in the DS80C400 User's Guide at 
 * http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf
 * 
 * \warning       This function is not multi-process safe.  If two processes
 *                try to call this function at the same time, its parameters
 *                may be destroyed, yielding unpredictable results.
 * 
 * \param     infinite  0 for non-infinite timeout, non-zero for infinite timeout (until woken)
 * \param     timeout   amount of time to sleep (if infinite==0)
 *
 * \return    0 for Success, non-zero for failure
 *
 * \sa        #task_threadiosleepnc
 * \sa        #task_threadresume
 */
//---------------------------------------------------------------------------
unsigned char  task_threadiosleep(unsigned char infinite, unsigned long timeout);

/**
 * \brief     Redirected function to put the current thread (which is already
 *            in a critical section) to sleep.
 *
 * This is a redirected function that should be used to put a thread to sleep, 
 * when the thread has already entered a critical section.  However, the DS80C400
 * does not support threads, so the default implementation of this function
 * puts the current task to sleep (which is assumed to be operating within
 * a critical section).
 * 
 * For more information on redirected functions, see the section <i>ROM Redirect 
 * Function Table</i> in the DS80C400 User's Guide at 
 * http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf
 * 
 * \warning       This function is not multi-process safe.  If two processes
 *                try to call this function at the same time, its parameters
 *                may be destroyed, yielding unpredictable results.
 * 
 * \param     infinite  0 for non-infinite timeout, non-zero for infinite timeout (until woken)
 * \param     timeout   amount of tuime to sleep (if infinite==0)
 *
 * \return    0 for Success, non-zero for failure
 *
 * \sa        #task_threadiosleep
 * \sa        #task_threadresume
 * \sa        #task_entercritsection
 */
//---------------------------------------------------------------------------
unsigned char  task_threadiosleepnc(unsigned char infinite, unsigned long timeout);


/**
 * \brief     Redirected function to save the state of the current thread
 *            in anticipation of a task/thread swap.
 *
 * This is a redirected function that should be used to save the state of the
 * current thread so it may be executed again later, after a call to 
 * <i>#task_threadrestore</i>.  However, the DS80C400 does not support threads, so
 * the default implementation of this function does nothing.
 *
 * For more information on redirected functions, see the section <i>ROM Redirect 
 * Function Table</i> in the DS80C400 User's Guide at 
 * http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf
 * 
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \sa         #task_threadrestore
 */
//---------------------------------------------------------------------------
void           task_threadsave(void);

/**
 * \brief     Redirected function to restore the state of a thread.
 *
 * This is a redirected function that should be used to restore the state of a
 * thread that was earlier saved with a call to <i>#task_threadsave</i>.  However, 
 * the DS80C400 does not support threads, so the default implementation of this 
 * function does nothing.
 *
 * For more information on redirected functions, see the section <i>ROM Redirect 
 * Function Table</i> in the DS80C400 User's Guide at 
 * http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf
 * 
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \sa         #task_threadsave
 */
//---------------------------------------------------------------------------
void           task_threadrestore(void);

////// Note: task_synch_sleep is deprecated use task_sleep
/// @cond
unsigned char  task_synch_sleep(unsigned int timeout_h, unsigned int timeout_l, unsigned char task);
/// @endcond

/** 
 * \brief     Redirected function to put a specified task to sleep for 
 *            a number of milliseconds.
 *
 * This is a redirected function that should be used to put a task
 * to sleep for some known period of time.  The default implementation of
 * this function calls the function <i>#task_wait</i>.
 * The task can be woken up prematurely using <i>#task_signal</i>.
 *
 * For more information on redirected functions, see the section <i>ROM Redirect 
 * Function Table</i> in the DS80C400 User's Guide at 
 * http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf
 * 
 * This function is safe to be called from multiple processes at the same 
 * time.
 * 
 * \param     task     task ID to put to sleep.  A value of zero means put the current
 *                     task to sleep.
 * \param     timeout  amount of time to put 'task' to sleep for
 *
 * \sa        #task_wait
 * \sa        #task_signal
 */
//---------------------------------------------------------------------------
unsigned char task_sleep(unsigned char task, long timeout);

/**
 * \brief     Redirected function to get the ID of the current task.
 *
 * This is a redirected function that should be used to get the process ID
 * of the current task.  The default implementation of this function calls
 * the function <i>#task_getcurrent</i>.
 * 
 * For more information on redirected functions, see the section <i>ROM Redirect 
 * Function Table</i> in the DS80C400 User's Guide at 
 * http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf
 * 
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \return    Task Id of the current task.
 * 
 * \sa        #task_getcurrent
 */
//---------------------------------------------------------------------------
unsigned char  task_gettaskid();

/**
 * \brief     Enters a critical section.
 *
 * Enters a critical section, which disallows process swapping until the
 * critical section is left.  Calls to <i>#task_entercritsection</i> should be
 * balanced with calls to <i>#task_leavecritsection</i> (or <i>#task_threadiosleepnc</i>).
 *
 * This function is safe to be called from multiple processes at the same
 * time.
 *
 * <b>NOTE:</b> An application should not stay in a critical section for extended
 * periods of time.  100-200us should be considered the maximum time.
 *
 * \sa        #task_leavecritsection
 * \sa        #task_threadiosleepnc
 */
//---------------------------------------------------------------------------
void           task_entercritsection(void);

/** 
 * \brief     Leaves a critical section.
 *
 * Leaves a critical section, which allows process swapping to continue.
 * Calls to <i>#task_leavecritsection</i> should have a matching call to
 * <i>#task_entercritsection</i>.
 *
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * <b>NOTE:</b> An application should not stay in a critical section for extended
 * periods of time.  100-200us should be considered the maximum time.
 *
 * \sa        #task_entercritsection
 * \sa        #task_threadiosleepnc
 */
//---------------------------------------------------------------------------
void           task_leavecritsection(void);

/** 
 * \brief     Gets the current reload value for the system's millisecond ticker.
 *
 * Gets the current reload value for the system's millisecond ticker.
 * When initialized, this reload value may not be correct for the system,
 * and calls to <i>#task_gettimemillis</i> may show the resulting inaccuracy
 * (for example, wall time may record 10 seconds while the DS80C400 
 * thinks 12 seconds have passes).  Use this function to verify the
 * system's current system millisecond ticker reload value.
 *
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \sa        #task_settickreload
 * \sa        #task_gettimemillis
 */
//---------------------------------------------------------------------------
unsigned int   task_gettickreload(void);

/** 
 * \brief     Sets the current reload value for the system's millisecond ticker.
 *
 * Sets the current reload value for the system's millisecond ticker.
 * When initialized, this reload value may not be correct for the system,
 * and calls to <i>#task_gettimemillis</i> may show the resulting inaccuracy
 * (for example, wall time may record 10 seconds while the DS80C400 
 * thinks 12 seconds have passes).  Use this function to set the
 * system's current system millisecond ticker reload value.
 *
 * This function is safe to be called from multiple processes at the same 
 * time.  This function should only be called after <i>#init_rom</i> has
 * been called.  If you do not have a 1-Wire device attached for MAC address
 * storage, you should call <i>#init_setclock</i> or <i>#init_setfrequency</i>
 * before calling <i>#init_rom</i> to initialize the system with a good clock 
 * reload value.
 *
 * \param     reload  New value for the system's millisecond reload timer.
 *                    Some reloads for common crystal frequencies include
 *                    #RELOAD_14_746, #RELOAD_18_432, #RELOAD_29_491, 
 *                    #RELOAD_36_864, #RELOAD_58_982, and #RELOAD_73_728.
 *                    Values for other crystals (and crystal settings) can 
 *                    also be used.  See the <a 
 *                    href="http://pdfserv.maxim-ic.com/arpdf/Design/DS80C400UG.pdf">
 *                    High Speed Microcontroller's User Guide</a> for more 
 *                    information on timers and timer settings.
 *
 * \sa        #init_setclock
 * \sa        #init_setfrequency
 * \sa        #task_gettickreload
 * \sa        #task_gettimemillis
 */
//---------------------------------------------------------------------------
void           task_settickreload(unsigned int reload);


/**
 * \brief        Returns the version number of this process scheduling library.
 *
 * This function is safe to be called from multiple processes at the same 
 * time.
 *
 * \return       Version number of this TASK library.
 */
//---------------------------------------------------------------------------
unsigned int task_version(void);


/**
 * \brief     Rerserves space on the reentrant stack.
 *
 * This function reserves the specified amount of space on the reentrant stack.
 * Any task that uses functions declared "reentrant" MUST call task_reentrant_stack.
 * <b>Note that space on the reentrant stack is NOT freed when a task is killed.</b>
 * An function called from an interrupt uses the reentrant stack of the foreground
 * process.
 * Note that the reentrant stack is part of XDATA. You can adjust the top of the
 * reentrant stack in startup400.a51.
 * 
 * \param     size  Amount (in bytes) to reserve on the reentrant stack.
 *
 * \return    -1 for failure (reentrant stack disabled in startup.a51), else the
 *            new lower bounds of the reentrant stack.
 *
 * \sa        #task_fork
 */
//---------------------------------------------------------------------------
void xdata *task_reentrant_stack(unsigned int size);

#pragma RESTORE
#endif
