/*---------------------------------------------------------------------------
 *  Copyright (C) 2003-2005 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 *
 * This file contains function definitions for the built-in ROM functions 
 * of the Dallas Semiconductor 400 processor.  This file is intended for use 
 * with the Keil MicroVision (uVision) C compiler.
 *
 * ---------------------------------------------------------------------------
 */
#ifndef __rom400_xnetstack_
#define __rom400_xnetstack_

/** \file rom400_xnetstack.h
 *  \brief Enhanced network stack for the DS80C400 ROM
 *
 *  This library contains a replacement network stack with better performance
 *  and more standards compliant functionality.  Since this library will replace
 *  the default ROM network stack, be careful of the physical location this 
 *  library.  If this library is targeted to reside in flash memory, your system
 *  will be limited by the speed of your flash.
 *
 *  To use this functionality, add xnetstack_install() to your program before
 *  calling init_rom and add the library to your build process.
 *  
 *  For detailed information on the DS80C400 please see the
 *  <a href="http://www.maxim-ic.com/products/microcontrollers/pdfs/network_microcontroller_supplement.pdf">
 *  High-Speed Microcontroller User's Guide: DS80C400 Supplement</a>.
 *
 */

/** Version number associated with this header file.  Should be the same as
 * the version number returned by the <i>#xnetstack_version</i> function.
 * \sa #xnetstack_version */
#define ROM400_XNETSTACK_VERSION         16

/**
 * \brief     Installs the enhanced network stack.
 *
 * This function installs the enhanced network stack functionality. The function
 * has to be called before init_rom().
 *
 */
//---------------------------------------------------------------------------
void xnetstack_install(void);

/**
 * \brief     Returns the version number of this library.
 *
 * \return    Version number of this library.
 */
//---------------------------------------------------------------------------
unsigned int xnetstack_version(void);


/**
 * \brief  Sets a factor to scale all TCP timeouts
 *
 * \param factor TCP scale factor (1 to 255, default: 32)
 *
 * \sa #xnetstack_get_tcptimeoutfactor
 */
//---------------------------------------------------------------------------
void xnetstack_set_tcptimeoutfactor(int factor);


/**
 * \brief  Gets the factor to scale all TCP timeouts
 *
 * \return TCP scale factor
 *
 * \sa #xnetstack_set_tcptimeoutfactor
 */
//---------------------------------------------------------------------------
int xnetstack_get_tcptimeoutfactor(void);

/**
 * \brief Enables/disables IPv6
 *
 * \param enable 1 to enable, 0 to disable
 *
 * <b>NOTE:</b> This function disables the IPv6 receiver and transmitter.
 * An application can still send packet to IPv6 addresses without receiving
 * an error message; these packets will be discarded at the driver level.
 */
//---------------------------------------------------------------------------
void xnetstack_set_ipv6(int enable);

/**
 * \brief Enables/disables ICMP echo replies
 *
 * \param enable 1 to enable, 0 to disable
 *
 * Setting this to 0 prevents the network stack from generating ICMP echo replies
 * (i.e. the device will no longer respond to "ping").
 */
//---------------------------------------------------------------------------
void xnetstack_set_icmpechoreplies(int enable);

/**
 * \brief Enables/disables ICMP destination unreachable messages
 *
 * \param enable 1 to enable, 0 to disable
 *
 * Setting this to 0 prevents the network stack from generating ICMP destination 
 * unreachables (i.e. the device will not respond when an unused port is accessed).
 */
//---------------------------------------------------------------------------
void xnetstack_set_icmpdestinationunreachable(int enable);

/**
 * \brief Sets the IGMP membership report type.
 *
 * \param type (0x12 for version 1, 0x16 for version 2)
 *
 * <b>NOTE:</b> This does not enable IGMPv2 compatibility, it merely changes
 * the type of membership reports to work around a problem with certain
 * switches. The default is IGMPv1.
 */
//---------------------------------------------------------------------------
void xnetstack_set_igmpreporttype(int type);

/**
 * \brief  Sets the ARP timeout.
 *
 * \param timeout ARP timeout (1 to 255, default: 16)
 *
 * This function manipulates the amount of timer ticks an ARP entry can
 * be pending (be unresolved) before the network stack considers a host
 * to be unreachable.
 *
 * Note that init_rom() resets the ARP timeout value.
 *
 * \sa #xnetstack_get_arptimeout
 */
//---------------------------------------------------------------------------
void xnetstack_set_arptimeout(int timeout);


/**
 * \brief  Gets the ARP timeout value.
 *
 * \return ARP timeout
 *
 * \sa #xnetstack_set_arptimeout
 */
//---------------------------------------------------------------------------
int xnetstack_get_arptimeout(void);

/**
 * \brief  Sets the number of ARP table entries.
 *
 * \param entries ARP table entries (16 to 127)
 *
 * \return 0 on success
 *
 * This function allocates the new ARP table from the kernel memory subsystem.
 * The memory allocated will be the table entry count * 12.
 * <b>NOTE:</b> Resizing the ARP table will destroy all entries, including
 * manually set static entries.
 *
 * \sa #xnetstack_set_arptablesize
 */
//---------------------------------------------------------------------------
int xnetstack_set_arptablesize(int entries);

/**
 * \brief  Gets the maximum number of ARP table entries.
 *
 * \return ARP table size
 *
 * \sa #xnetstack_set_arptimeout
 */
//---------------------------------------------------------------------------
int xnetstack_get_arptablesize(void);

/**
 * \brief  Sets a protocol filter for the RAW socket.
 *
 * \param proto Ethernet protocol (e.g. 0x0800 for IPv4)
 *
 * This function reduces the system load when a raw socket is used by filtering
 * for a given Ethernet protocol type at the Ethernet driver layer.
 * Note: The network stack only supports Ethernet II frames.
 *
 * \sa #xnetstack_disable_rawfilter
 */
//---------------------------------------------------------------------------
void xnetstack_set_rawfilter(unsigned int proto);

/**
 * \brief  Disables the protocol filter for the RAW socket.
 *
 * This function disables the filter set by xnetstack_set_rawfilter.
 *
 * \sa #xnetstack_set_rawfilter
 */
//---------------------------------------------------------------------------
void xnetstack_disable_rawfilter(void);

/** Argument to function <i>#socket</i> to create a RAW socket (same as <i>#SOCK_RAW</i>)
 * \sa #socket  */
#define SOCKET_TYPE_RAW     2
/** Argument to function <i>#socket</i> to create a RAW socket (same as <i>#SOCKET_TYPE_RAW</i>)
 * \sa #socket  */
#define SOCK_RAW            2

/**
 * \brief  Sets whether the MII interface should be used to talk to
 * the physical network interface chip (PHY).
 *
 * \param value <i>#MDIO_ENABLE</i>, <i>#MDIO_DISABLE_HDX</i>, <i>#MDIO_DISABLE_FDX</i>
 *
 * This function enables / disables MII communication over the MII interface.
 * MII communication with the physical interface is used to determine link availability
 * and status.
 * When disabling MDIO, you must specify whether the link is half duplex or
 * full duplex.
 * This function should be called after <i>#xnetstack_install</i>, but before
 * <i>#init_rom</i>.
 *
 */
//---------------------------------------------------------------------------
void xnetstack_set_mdio(int value);

/** Argument to function <i>#xnetstack_set_mdio</i> to enable MDIO link detection.
 * \sa #xnetstack_set_mdio  */
#define MDIO_ENABLE         0
/** Argument to function <i>#xnetstack_set_mdio</i> to disable MDIO link detection
 * and to force the link to half duplex.
 * \sa #xnetstack_set_mdio  */
#define MDIO_DISABLE_HDX    1
/** Argument to function <i>#xnetstack_set_mdio</i> to disable MDIO link detection
 * and to force the link to full duplex.
 * \sa #xnetstack_set_mdio  */
#define MDIO_DISABLE_FDX    2

/**
 * \brief Enables/disables inbound IGMP processing.
 *
 * \param enable 1 to enable, 0 to disable
 *
 * <b>NOTE:</b> This function disables the IGMP receiver and feeds all inbound
 * packets to the raw packet queue. If there is no raw socket, or if the raw
 * filter doesn't match, the packets will be discarded.
 */
//---------------------------------------------------------------------------
void xnetstack_set_igmp(int enable);

#endif
